# Copyright (c) 2021 Wenet Community. (authors: Binbin Zhang)
#               2023 Wenet Community. (authors: Dinghao Zhou)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from functools import partial
from typing import Optional
from wenet.dataset import processor
from wenet.dataset.datapipes import (WenetRawDatasetSource,
                                     WenetTarShardDatasetSource)
from wenet.text.base_tokenizer import BaseTokenizer
from wenet.utils.file_utils import read_symbol_table


def Dataset(data_type,
            data_list_file,
            tokenizer: Optional[BaseTokenizer] = None,
            conf=None,
            partition=True):
    """ Construct dataset from arguments

        We have two shuffle stage in the Dataset. The first is global
        shuffle at shards tar/raw file level. The second is global shuffle
        at training samples level.

        Args:
            data_type(str): raw/shard
            tokenizer (BaseTokenizer or None): tokenizer to tokenize
            partition(bool): whether to do data partition in terms of rank
    """
    assert conf is not None
    assert data_type in ['raw', 'shard']
    # cycle dataset
    cycle = conf.get('cycle', 1)
    if data_type == 'raw':
        dataset = WenetRawDatasetSource(data_list_file,
                                        partition=partition,
                                        cycle=cycle)
        dataset = dataset.map(processor.parse_json)
    else:
        dataset = WenetTarShardDatasetSource(data_list_file,
                                             partition=partition,
                                             cycle=cycle)
    dataset = dataset.map_ignore_error(processor.decode_wav)

    speaker_conf = conf.get('speaker_conf', None)
    if speaker_conf is not None:
        assert 'speaker_table_path' in speaker_conf
        speaker_table = read_symbol_table(speaker_conf['speaker_table_path'])
        dataset = dataset.map(
            partial(processor.parse_speaker, speaker_dict=speaker_table))

    if tokenizer is not None:
        dataset = dataset.map(partial(processor.tokenize, tokenizer=tokenizer))

    filter_conf = conf.get('filter_conf', {})
    dataset = dataset.filter(partial(processor.filter, **filter_conf))

    resample_conf = conf.get('resample_conf', {})
    dataset = dataset.map(partial(processor.resample, **resample_conf))

    speed_perturb = conf.get('speed_perturb', False)
    if speed_perturb:
        dataset = dataset.map(partial(processor.speed_perturb))

    feats_type = conf.get('feats_type', 'fbank')
    assert feats_type in ['fbank', 'mfcc', 'log_mel_spectrogram']
    if feats_type == 'fbank':
        fbank_conf = conf.get('fbank_conf', {})
        dataset = dataset.map(partial(processor.compute_fbank, **fbank_conf))
    elif feats_type == 'mfcc':
        mfcc_conf = conf.get('mfcc_conf', {})
        dataset = dataset.map(partial(processor.compute_mfcc, **mfcc_conf))
    elif feats_type == 'log_mel_spectrogram':
        log_mel_spectrogram_conf = conf.get('log_mel_spectrogram_conf', {})
        dataset = dataset.map(
            partial(processor.compute_log_mel_spectrogram,
                    **log_mel_spectrogram_conf))
    spec_aug = conf.get('spec_aug', True)
    spec_sub = conf.get('spec_sub', False)
    spec_trim = conf.get('spec_trim', False)
    if spec_aug:
        spec_aug_conf = conf.get('spec_aug_conf', {})
        dataset = dataset.map(partial(processor.spec_aug, **spec_aug_conf))
    if spec_sub:
        spec_sub_conf = conf.get('spec_sub_conf', {})
        dataset = dataset.map(partial(processor.spec_sub, **spec_sub_conf))
    if spec_trim:
        spec_trim_conf = conf.get('spec_trim_conf', {})
        dataset = dataset.map(partial(processor.spec_trim, **spec_trim_conf))

    language_conf = conf.get('language_conf', {"limited_langs": ['zh', 'en']})
    dataset = dataset.map(partial(processor.detect_language, **language_conf))
    dataset = dataset.map(processor.detect_task)

    shuffle = conf.get('shuffle', True)
    if shuffle:
        shuffle_conf = conf.get('shuffle_conf', {})
        dataset = dataset.shuffle(buffer_size=shuffle_conf['shuffle_size'])

    sort = conf.get('sort', True)
    if sort:
        sort_conf = conf.get('sort_conf', {})
        dataset = dataset.sort(buffer_size=sort_conf['sort_size'],
                               key_func=processor.sort_by_feats)

    batch_conf = conf.get('batch_conf', {})
    batch_type = batch_conf.get('batch_type', 'static')
    assert batch_type in ['static', 'bucket', 'dynamic']
    if batch_type == 'static':
        assert 'batch_size' in batch_conf
        batch_size = batch_conf.get('batch_size', 16)
        dataset = dataset.batch(batch_size, wrapper_class=processor.padding)
    elif batch_type == 'bucket':
        assert 'bucket_boundaries' in batch_conf
        assert 'bucket_batch_sizes' in batch_conf
        dataset = dataset.bucket_by_sequence_length(
            processor.feats_length_fn,
            batch_conf['bucket_boundaries'],
            batch_conf['bucket_batch_sizes'],
            wrapper_class=processor.padding)
    else:
        max_frames_in_batch = batch_conf.get('max_frames_in_batch', 12000)
        dataset = dataset.dynamic_batch(
            processor.DynamicBatchWindow(max_frames_in_batch),
            wrapper_class=processor.padding,
        )

    return dataset



# Copyright (c) 2021 Mobvoi Inc (Binbin Zhang, Di Wu)
#               2022 Xingchen Song (sxc19@mails.tsinghua.edu.cn)
#               2022 58.com(Wuba) Inc AI Lab.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# Modified from EfficientConformer(https://github.com/burchim/EfficientConformer)
#               Paper(https://arxiv.org/abs/2109.01163)
"""Encoder definition."""
from typing import Tuple, Optional, List, Union

import torch
import logging
import torch.nn.functional as F

from wenet.transformer.positionwise_feed_forward import PositionwiseFeedForward
from wenet.transformer.encoder_layer import ConformerEncoderLayer

from wenet.efficient_conformer.convolution import ConvolutionModule
from wenet.efficient_conformer.encoder_layer import StrideConformerEncoderLayer

from wenet.utils.mask import make_pad_mask
from wenet.utils.mask import add_optional_chunk_mask
from wenet.utils.class_utils import (
    WENET_ATTENTION_CLASSES,
    WENET_EMB_CLASSES,
    WENET_SUBSAMPLE_CLASSES,
    WENET_ACTIVATION_CLASSES,
)


class EfficientConformerEncoder(torch.nn.Module):
    """Conformer encoder module."""

    def __init__(self,
                 input_size: int,
                 output_size: int = 256,
                 attention_heads: int = 4,
                 linear_units: int = 2048,
                 num_blocks: int = 6,
                 dropout_rate: float = 0.1,
                 positional_dropout_rate: float = 0.1,
                 attention_dropout_rate: float = 0.0,
                 input_layer: str = "conv2d",
                 pos_enc_layer_type: str = "rel_pos",
                 normalize_before: bool = True,
                 static_chunk_size: int = 0,
                 use_dynamic_chunk: bool = False,
                 global_cmvn: torch.nn.Module = None,
                 use_dynamic_left_chunk: bool = False,
                 macaron_style: bool = True,
                 activation_type: str = "swish",
                 use_cnn_module: bool = True,
                 cnn_module_kernel: int = 15,
                 causal: bool = False,
                 cnn_module_norm: str = "batch_norm",
                 stride_layer_idx: Optional[Union[int, List[int]]] = 3,
                 stride: Optional[Union[int, List[int]]] = 2,
                 group_layer_idx: Optional[Union[int, List[int],
                                                 tuple]] = (0, 1, 2, 3),
                 group_size: int = 3,
                 stride_kernel: bool = True,
                 **kwargs):
        """Construct Efficient Conformer Encoder

        Args:
            input_size to use_dynamic_chunk, see in BaseEncoder
            macaron_style (bool): Whether to use macaron style for
                positionwise layer.
            activation_type (str): Encoder activation function type.
            use_cnn_module (bool): Whether to use convolution module.
            cnn_module_kernel (int): Kernel size of convolution module.
            causal (bool): whether to use causal convolution or not.
            stride_layer_idx (list): layer id with StrideConv, start from 0
            stride (list): stride size of each StrideConv in efficient conformer
            group_layer_idx (list): layer id with GroupedAttention, start from 0
            group_size (int): group size of every GroupedAttention layer
            stride_kernel (bool): default True. True: recompute cnn kernels with stride.
        """
        super().__init__()
        self._output_size = output_size

        logging.info(
            f"input_layer = {input_layer}, "
            f"subsampling_class = {WENET_SUBSAMPLE_CLASSES[input_layer]}")

        self.global_cmvn = global_cmvn
        self.embed = WENET_SUBSAMPLE_CLASSES[input_layer](
            input_size,
            output_size,
            dropout_rate,
            WENET_EMB_CLASSES[pos_enc_layer_type](output_size,
                                                  positional_dropout_rate),
        )
        self.input_layer = input_layer
        self.normalize_before = normalize_before
        self.after_norm = torch.nn.LayerNorm(output_size, eps=1e-5)
        self.static_chunk_size = static_chunk_size
        self.use_dynamic_chunk = use_dynamic_chunk
        self.use_dynamic_left_chunk = use_dynamic_left_chunk

        activation = WENET_ACTIVATION_CLASSES[activation_type]()
        self.num_blocks = num_blocks
        self.attention_heads = attention_heads
        self.cnn_module_kernel = cnn_module_kernel
        self.global_chunk_size = 0
        self.chunk_feature_map = 0

        # efficient conformer configs
        self.stride_layer_idx = [stride_layer_idx] \
            if type(stride_layer_idx) == int else stride_layer_idx
        self.stride = [stride] \
            if type(stride) == int else stride
        self.group_layer_idx = [group_layer_idx] \
            if type(group_layer_idx) == int else group_layer_idx
        self.grouped_size = group_size  # group size of every GroupedAttention layer

        assert len(self.stride) == len(self.stride_layer_idx)
        self.cnn_module_kernels = [cnn_module_kernel
                                   ]  # kernel size of each StridedConv
        for i in self.stride:
            if stride_kernel:
                self.cnn_module_kernels.append(self.cnn_module_kernels[-1] //
                                               i)
            else:
                self.cnn_module_kernels.append(self.cnn_module_kernels[-1])

        logging.info(f"stride_layer_idx= {self.stride_layer_idx}, "
                     f"stride = {self.stride}, "
                     f"cnn_module_kernel = {self.cnn_module_kernels}, "
                     f"group_layer_idx = {self.group_layer_idx}, "
                     f"grouped_size = {self.grouped_size}")

        # feed-forward module definition
        positionwise_layer = PositionwiseFeedForward
        positionwise_layer_args = (
            output_size,
            linear_units,
            dropout_rate,
            activation,
        )
        # convolution module definition
        convolution_layer = ConvolutionModule

        # encoder definition
        index = 0
        layers = []
        for i in range(num_blocks):
            # self-attention module definition
            if i in self.group_layer_idx:
                encoder_selfattn_layer = WENET_ATTENTION_CLASSES[
                    "grouped_rel_selfattn"]
                encoder_selfattn_layer_args = (attention_heads, output_size,
                                               attention_dropout_rate,
                                               self.grouped_size)
            else:
                if pos_enc_layer_type == "no_pos":
                    encoder_selfattn_layer = WENET_ATTENTION_CLASSES[
                        "selfattn"]
                else:
                    encoder_selfattn_layer = WENET_ATTENTION_CLASSES[
                        "rel_selfattn"]
                encoder_selfattn_layer_args = (attention_heads, output_size,
                                               attention_dropout_rate)

            # conformer module definition
            if i in self.stride_layer_idx:
                # conformer block with downsampling
                convolution_layer_args_stride = (
                    output_size, self.cnn_module_kernels[index], activation,
                    cnn_module_norm, causal, True, self.stride[index])
                layers.append(
                    StrideConformerEncoderLayer(
                        output_size,
                        encoder_selfattn_layer(*encoder_selfattn_layer_args),
                        positionwise_layer(*positionwise_layer_args),
                        positionwise_layer(*positionwise_layer_args)
                        if macaron_style else None,
                        convolution_layer(*convolution_layer_args_stride)
                        if use_cnn_module else None,
                        torch.nn.AvgPool1d(
                            kernel_size=self.stride[index],
                            stride=self.stride[index],
                            padding=0,
                            ceil_mode=True,
                            count_include_pad=False),  # pointwise_conv_layer
                        dropout_rate,
                        normalize_before,
                    ))
                index = index + 1
            else:
                # conformer block
                convolution_layer_args_normal = (
                    output_size, self.cnn_module_kernels[index], activation,
                    cnn_module_norm, causal)
                layers.append(
                    ConformerEncoderLayer(
                        output_size,
                        encoder_selfattn_layer(*encoder_selfattn_layer_args),
                        positionwise_layer(*positionwise_layer_args),
                        positionwise_layer(*positionwise_layer_args)
                        if macaron_style else None,
                        convolution_layer(*convolution_layer_args_normal)
                        if use_cnn_module else None,
                        dropout_rate,
                        normalize_before,
                    ))

        self.encoders = torch.nn.ModuleList(layers)

    def set_global_chunk_size(self, chunk_size):
        """Used in ONNX export.
        """
        logging.info(f"set global chunk size: {chunk_size}, default is 0.")
        self.global_chunk_size = chunk_size
        if self.embed.subsampling_rate == 2:
            self.chunk_feature_map = 2 * self.global_chunk_size + 1
        elif self.embed.subsampling_rate == 6:
            self.chunk_feature_map = 6 * self.global_chunk_size + 5
        elif self.embed.subsampling_rate == 8:
            self.chunk_feature_map = 8 * self.global_chunk_size + 7
        else:
            self.chunk_feature_map = 4 * self.global_chunk_size + 3

    def output_size(self) -> int:
        return self._output_size

    def calculate_downsampling_factor(self, i: int) -> int:
        factor = 1
        for idx, stride_idx in enumerate(self.stride_layer_idx):
            if i > stride_idx:
                factor *= self.stride[idx]
        return factor

    def forward(
        self,
        xs: torch.Tensor,
        xs_lens: torch.Tensor,
        decoding_chunk_size: int = 0,
        num_decoding_left_chunks: int = -1,
    ) -> Tuple[torch.Tensor, torch.Tensor]:
        """Embed positions in tensor.
        Args:
            xs: padded input tensor (B, T, D)
            xs_lens: input length (B)
            decoding_chunk_size: decoding chunk size for dynamic chunk
                0: default for training, use random dynamic chunk.
                <0: for decoding, use full chunk.
                >0: for decoding, use fixed chunk size as set.
            num_decoding_left_chunks: number of left chunks, this is for decoding,
            the chunk size is decoding_chunk_size.
                >=0: use num_decoding_left_chunks
                <0: use all left chunks
        Returns:
            encoder output tensor xs, and subsampled masks
            xs: padded output tensor (B, T' ~= T/subsample_rate, D)
            masks: torch.Tensor batch padding mask after subsample
                (B, 1, T' ~= T/subsample_rate)
        """
        T = xs.size(1)
        masks = ~make_pad_mask(xs_lens, T).unsqueeze(1)  # (B, 1, T)
        if self.global_cmvn is not None:
            xs = self.global_cmvn(xs)
        xs, pos_emb, masks = self.embed(xs, masks)
        mask_pad = masks  # (B, 1, T/subsample_rate)
        chunk_masks = add_optional_chunk_mask(xs, masks,
                                              self.use_dynamic_chunk,
                                              self.use_dynamic_left_chunk,
                                              decoding_chunk_size,
                                              self.static_chunk_size,
                                              num_decoding_left_chunks)
        index = 0  # traverse stride
        for i, layer in enumerate(self.encoders):
            # layer return : x, mask, new_att_cache, new_cnn_cache
            xs, chunk_masks, _, _ = layer(xs, chunk_masks, pos_emb, mask_pad)
            if i in self.stride_layer_idx:
                masks = masks[:, :, ::self.stride[index]]
                chunk_masks = chunk_masks[:, ::self.stride[index], ::self.
                                          stride[index]]
                mask_pad = masks
                pos_emb = pos_emb[:, ::self.stride[index], :]
                index = index + 1

        if self.normalize_before:
            xs = self.after_norm(xs)
        # Here we assume the mask is not changed in encoder layers, so just
        # return the masks before encoder layers, and the masks will be used
        # for cross attention with decoder later
        return xs, masks

    def forward_chunk(
        self,
        xs: torch.Tensor,
        offset: int,
        required_cache_size: int,
        att_cache: torch.Tensor = torch.zeros(0, 0, 0, 0),
        cnn_cache: torch.Tensor = torch.zeros(0, 0, 0, 0),
        att_mask: torch.Tensor = torch.ones((0, 0, 0), dtype=torch.bool)
    ) -> Tuple[torch.Tensor, torch.Tensor, torch.Tensor]:
        """ Forward just one chunk

        Args:
            xs (torch.Tensor): chunk input
            offset (int): current offset in encoder output time stamp
            required_cache_size (int): cache size required for next chunk
                compuation
                >=0: actual cache size
                <0: means all history cache is required
            att_cache (torch.Tensor): cache tensor for KEY & VALUE in
                transformer/conformer attention, with shape
                (elayers, head, cache_t1, d_k * 2), where
                `head * d_k == hidden-dim` and
                `cache_t1 == chunk_size * num_decoding_left_chunks`.
            cnn_cache (torch.Tensor): cache tensor for cnn_module in conformer,
                (elayers, b=1, hidden-dim, cache_t2), where
                `cache_t2 == cnn.lorder - 1`
            att_mask : mask matrix of self attention

        Returns:
            torch.Tensor: output of current input xs
            torch.Tensor: subsampling cache required for next chunk computation
            List[torch.Tensor]: encoder layers output cache required for next
                chunk computation
            List[torch.Tensor]: conformer cnn cache

        """
        assert xs.size(0) == 1

        # using downsampling factor to recover offset
        offset *= self.calculate_downsampling_factor(self.num_blocks + 1)

        chunk_masks = torch.ones(1,
                                 xs.size(1),
                                 device=xs.device,
                                 dtype=torch.bool)
        chunk_masks = chunk_masks.unsqueeze(1)  # (1, 1, xs-time)

        real_len = 0
        if self.global_chunk_size > 0:
            # for ONNX decode simulation， padding xs to chunk_size
            real_len = xs.size(1)
            pad_len = self.chunk_feature_map - real_len
            xs = F.pad(xs, (0, 0, 0, pad_len), value=0.0)
            chunk_masks = F.pad(chunk_masks, (0, pad_len), value=0.0)

        if self.global_cmvn is not None:
            xs = self.global_cmvn(xs)

        # NOTE(xcsong): Before embed, shape(xs) is (b=1, time, mel-dim)
        xs, pos_emb, chunk_masks = self.embed(xs, chunk_masks, offset)
        elayers, cache_t1 = att_cache.size(0), att_cache.size(2)
        chunk_size = xs.size(1)
        attention_key_size = cache_t1 + chunk_size
        # NOTE(xcsong): After  embed, shape(xs) is (b=1, chunk_size, hidden-dim)
        # shape(pos_emb) = (b=1, chunk_size, emb_size=output_size=hidden-dim)

        if required_cache_size < 0:
            next_cache_start = 0
        elif required_cache_size == 0:
            next_cache_start = attention_key_size
        else:
            next_cache_start = max(attention_key_size - required_cache_size, 0)

        r_att_cache = []
        r_cnn_cache = []
        mask_pad = torch.ones(1,
                              xs.size(1),
                              device=xs.device,
                              dtype=torch.bool)
        mask_pad = mask_pad.unsqueeze(1)  # batchPad (b=1, 1, time=chunk_size)

        if self.global_chunk_size > 0:
            # for ONNX decode simulation
            pos_emb = self.embed.position_encoding(
                offset=max(offset - cache_t1, 0),
                size=cache_t1 + self.global_chunk_size)
            att_mask[:, :, -self.global_chunk_size:] = chunk_masks
            mask_pad = chunk_masks.to(torch.bool)
        else:
            pos_emb = self.embed.position_encoding(offset=offset - cache_t1,
                                                   size=attention_key_size)

        max_att_len, max_cnn_len = 0, 0  # for repeat_interleave of new_att_cache
        for i, layer in enumerate(self.encoders):
            factor = self.calculate_downsampling_factor(i)
            # NOTE(xcsong): Before layer.forward
            #   shape(att_cache[i:i + 1]) is (1, head, cache_t1, d_k * 2),
            #   shape(cnn_cache[i])       is (b=1, hidden-dim, cache_t2)
            # shape(new_att_cache) = [ batch, head, time2, outdim//head * 2 ]
            att_cache_trunc = 0
            if xs.size(1) + att_cache.size(2) / factor > pos_emb.size(1):
                # The time step is not divisible by the downsampling multiple
                att_cache_trunc = xs.size(1) + \
                    att_cache.size(2) // factor - pos_emb.size(1) + 1
            xs, _, new_att_cache, new_cnn_cache = layer(
                xs,
                att_mask,
                pos_emb,
                mask_pad=mask_pad,
                att_cache=att_cache[i:i +
                                    1, :, ::factor, :][:, :,
                                                       att_cache_trunc:, :],
                cnn_cache=cnn_cache[i, :, :, :]
                if cnn_cache.size(0) > 0 else cnn_cache)

            if i in self.stride_layer_idx:
                # compute time dimension for next block
                efficient_index = self.stride_layer_idx.index(i)
                att_mask = att_mask[:, ::self.stride[efficient_index], ::self.
                                    stride[efficient_index]]
                mask_pad = mask_pad[:, ::self.stride[efficient_index], ::self.
                                    stride[efficient_index]]
                pos_emb = pos_emb[:, ::self.stride[efficient_index], :]

            # shape(new_att_cache) = [batch, head, time2, outdim]
            new_att_cache = new_att_cache[:, :, next_cache_start // factor:, :]
            # shape(new_cnn_cache) = [1, batch, outdim, cache_t2]
            new_cnn_cache = new_cnn_cache.unsqueeze(0)

            # use repeat_interleave to new_att_cache
            new_att_cache = new_att_cache.repeat_interleave(repeats=factor,
                                                            dim=2)
            # padding new_cnn_cache to cnn.lorder for casual convolution
            new_cnn_cache = F.pad(
                new_cnn_cache,
                (self.cnn_module_kernel - 1 - new_cnn_cache.size(3), 0))

            if i == 0:
                # record length for the first block as max length
                max_att_len = new_att_cache.size(2)
                max_cnn_len = new_cnn_cache.size(3)

            # update real shape of att_cache and cnn_cache
            r_att_cache.append(new_att_cache[:, :, -max_att_len:, :])
            r_cnn_cache.append(new_cnn_cache[:, :, :, -max_cnn_len:])

        if self.normalize_before:
            xs = self.after_norm(xs)

        # NOTE(xcsong): shape(r_att_cache) is (elayers, head, ?, d_k * 2),
        #   ? may be larger than cache_t1, it depends on required_cache_size
        r_att_cache = torch.cat(r_att_cache, dim=0)
        # NOTE(xcsong): shape(r_cnn_cache) is (e, b=1, hidden-dim, cache_t2)
        r_cnn_cache = torch.cat(r_cnn_cache, dim=0)

        if self.global_chunk_size > 0 and real_len:
            chunk_real_len = real_len // self.embed.subsampling_rate // \
                self.calculate_downsampling_factor(self.num_blocks + 1)
            # Keeping 1 more timestep can mitigate information leakage
            #   from the encoder caused by the padding
            xs = xs[:, :chunk_real_len + 1, :]

        return xs, r_att_cache, r_cnn_cache

    def forward_chunk_by_chunk(
            self,
            xs: torch.Tensor,
            decoding_chunk_size: int,
            num_decoding_left_chunks: int = -1,
            use_onnx=False) -> Tuple[torch.Tensor, torch.Tensor]:
        """ Forward input chunk by chunk with chunk_size like a streaming
            fashion

        Here we should pay special attention to computation cache in the
        streaming style forward chunk by chunk. Three things should be taken
        into account for computation in the current network:
            1. transformer/conformer encoder layers output cache
            2. convolution in conformer
            3. convolution in subsampling

        However, we don't implement subsampling cache for:
            1. We can control subsampling module to output the right result by
               overlapping input instead of cache left context, even though it
               wastes some computation, but subsampling only takes a very
               small fraction of computation in the whole model.
            2. Typically, there are several covolution layers with subsampling
               in subsampling module, it is tricky and complicated to do cache
               with different convolution layers with different subsampling
               rate.
            3. Currently, nn.Sequential is used to stack all the convolution
               layers in subsampling, we need to rewrite it to make it work
               with cache, which is not prefered.
        Args:
            xs (torch.Tensor): (1, max_len, dim)
            decoding_chunk_size (int): decoding chunk size
            num_decoding_left_chunks (int):
            use_onnx (bool): True for simulating ONNX model inference.
        """
        assert decoding_chunk_size > 0
        # The model is trained by static or dynamic chunk
        assert self.static_chunk_size > 0 or self.use_dynamic_chunk
        subsampling = self.embed.subsampling_rate
        context = self.embed.right_context + 1  # Add current frame
        stride = subsampling * decoding_chunk_size
        decoding_window = (decoding_chunk_size - 1) * subsampling + context
        num_frames = xs.size(1)

        outputs = []
        offset = 0
        required_cache_size = decoding_chunk_size * num_decoding_left_chunks
        if use_onnx:
            logging.info("Simulating for ONNX runtime ...")
            att_cache: torch.Tensor = torch.zeros(
                (self.num_blocks, self.attention_heads, required_cache_size,
                 self.output_size() // self.attention_heads * 2),
                device=xs.device)
            cnn_cache: torch.Tensor = torch.zeros(
                (self.num_blocks, 1, self.output_size(),
                 self.cnn_module_kernel - 1),
                device=xs.device)
            self.set_global_chunk_size(chunk_size=decoding_chunk_size)
        else:
            logging.info("Simulating for JIT runtime ...")
            att_cache: torch.Tensor = torch.zeros((0, 0, 0, 0),
                                                  device=xs.device)
            cnn_cache: torch.Tensor = torch.zeros((0, 0, 0, 0),
                                                  device=xs.device)

        # Feed forward overlap input step by step
        for cur in range(0, num_frames - context + 1, stride):
            end = min(cur + decoding_window, num_frames)
            logging.info(f"-->> frame chunk msg: cur={cur}, "
                         f"end={end}, num_frames={end-cur}, "
                         f"decoding_window={decoding_window}")
            if use_onnx:
                att_mask: torch.Tensor = torch.ones(
                    (1, 1, required_cache_size + decoding_chunk_size),
                    dtype=torch.bool,
                    device=xs.device)
                if cur == 0:
                    att_mask[:, :, :required_cache_size] = 0
            else:
                att_mask: torch.Tensor = torch.ones((0, 0, 0),
                                                    dtype=torch.bool,
                                                    device=xs.device)

            chunk_xs = xs[:, cur:end, :]
            (y, att_cache, cnn_cache) = \
                self.forward_chunk(
                    chunk_xs, offset, required_cache_size,
                    att_cache, cnn_cache, att_mask)
            outputs.append(y)
            offset += y.size(1)

        ys = torch.cat(outputs, 1)
        masks = torch.ones(1,
                           1,
                           ys.size(1),
                           device=ys.device,
                           dtype=torch.bool)
        return ys, masks



import pandas as pd


def do_read_and_write_xlsx(file_path):
    # 读取 Excel 文件
    xls = pd.ExcelFile(file_path)

    # 读取工作表1的第一列和第二列
    sheet1 = pd.read_excel(xls, sheet_name=xls.sheet_names[0])
    column1_sheet1 = sheet1.iloc[:, 0]
    column2_sheet1 = sheet1.iloc[:, 1]

    # 将工作表1的第一列和第二列分别写入A.txt和B.txt
    with open('./A.txt', 'w', encoding='utf-8') as file_a:
        for item in column1_sheet1:
            file_a.write(f"{item}\n")

    with open('./B.txt', 'w', encoding='utf-8') as file_b:
        for item in column2_sheet1:
            file_b.write(f"{item}\n")

    # 读取工作表2的第一列
    sheet2 = pd.read_excel(xls, sheet_name=xls.sheet_names[1])
    column1_sheet2 = sheet2.iloc[:, 0]

    # 将工作表2的第一列写入C.txt
    with open('./C.txt', 'w', encoding='utf-8') as file_c:
        for item in column1_sheet2:
            file_c.write(f"{item}\n")

    # 读取工作表3的第一列
    sheet3 = pd.read_excel(xls, sheet_name=xls.sheet_names[2])
    column1_sheet3 = sheet3.iloc[:, 0]

    # 将工作表3的第一列写入D.txt
    with open('./D.txt', 'w', encoding='utf-8') as file_d:
        for item in column1_sheet3:
            file_d.write(f"{item}\n")

if __name__ == '__main__':
    # 调用函数，并传入 xlsx 文件的路径
    file_path = "./youtube_url.xlsx"
    do_read_and_write_xlsx(file_path)



"""
为wenet训练框架加入musan噪音增强功能
"""
import os
import random
import sys

import torch
import torchaudio
from gxl_ai_utils.utils.utils_file import global_timer

sys.path.insert(0, '../../../')
from gxl_ai_utils.utils import utils_file

# 先把musan音频搬运到nfs15
def do_copy_musan():
    source_dir = "/home/work_nfs8/asr_data/data/musan"
    partions = ["music", "noise"]
    output_dir = '/home/work_nfs15/asr_data/data/musan'
    utils_file.makedir_sil(output_dir)
    for partion in partions:
        print("processing {}".format(partion))
        output_dir_tmp = os.path.join(output_dir, partion)
        source_dir_tmp = os.path.join(source_dir, partion)
        utils_file.makedir_sil(output_dir_tmp)
        # 得到wav.scp
        wav_scp_path = os.path.join(output_dir_tmp, 'wav.scp')
        wav_dict = utils_file.get_scp_for_wav_dir(source_dir_tmp, recursive=True)
        new_wav_dict_for_write = {}
        for key, value in wav_dict.items():
            wav_name_with_suffix = os.path.basename(value)
            new_value = os.path.join(output_dir_tmp, wav_name_with_suffix)
            new_wav_dict_for_write[key] = new_value
        utils_file.write_dict_to_scp(new_wav_dict_for_write, wav_scp_path)
        # 开始复制数据
        num_thread = 20
        runner = utils_file.GxlFixedProcessPool(num_thread)
        wav_dict_list = utils_file.do_split_dict(wav_dict, num_thread)
        for i, wav_dict_item in enumerate(wav_dict_list):
            runner.add_thread(utils_file.little_func_for_cp_from_dict, [wav_dict_item, output_dir_tmp, i])
        runner.start()
def do_fix_scp_wav():
    source_dir = "/home/work_nfs15/asr_data/data/musan"
    partions = ["music", "noise"]
    for partion in partions:
        wav_scp_path = os.path.join(source_dir, partion, "wav.scp")
        wav_dict = utils_file.load_dict_from_scp(wav_scp_path)
        new_wav_dict = {}
        for key, value in wav_dict.items():
            new_wav_dict[key] = value +'.wav'
        utils_file.write_dict_to_scp(new_wav_dict, wav_scp_path)


# 得到对干净语音进行增强的函数
def do_noice_augment(input_clear_wav:torch.Tensor, input_noice_wav:torch.Tensor, snr:float=5):
    """
    信噪比具体情况如下：
    30： 隐约能听到背景噪音。
    15： 还是只能感受到最后的呲呲声音
    10： 开始能听到真正的敲门声音
    -30： 人话声有些小了
    -40： 隐约有点说话声音
    -45： 主要是噪音，基本没法听到人声
    训练的时候推荐从30到-30
    AI给出的比例：
    20-30： 10%  安静的图书馆或自习室 乡村户外安静的自然环境
    10-20: 35% 普通的室内办公室环境 街边的咖啡馆或餐厅
    0-10: 40%  热闹的商场或超市 正在行驶的公交车或地铁内
    -10 -0: 10% 嘈杂的工厂车间 喧闹的酒吧或夜总会
    -30 - -10: 5% 大型建筑工地  飞机起飞或降落的跑道附近

    :param input_clear_wav:(1,samples)
    :param input_noice_wav:(1,samples)
    :param snr: 信噪比
    :return:
    """
    while input_noice_wav.size(1) < input_clear_wav.size(1):
        input_noice_wav = torch.cat([input_noice_wav, input_noice_wav], dim=1)

    # 先计算语音的功率（这里明确在样本数量维度求平均来近似功率，保持维度方便后续计算）
    clean_power = torch.mean(input_clear_wav ** 2, dim=1, keepdim=True)
    # 计算噪声的功率，同样在对应维度求平均
    noise_power = torch.mean(input_noice_wav ** 2, dim=1, keepdim=True)

    # 根据信噪比公式推导出噪声需要缩放的因子
    scale_factor = torch.sqrt(clean_power / (noise_power * (10 ** (snr / 10))))
    # 先按照干净语音的长度裁剪噪声张量
    input_noice_wav = input_noice_wav[:, :input_clear_wav.shape[1]]
    # 对裁剪后的噪声张量进行缩放，以达到期望的信噪比
    scaled_noise_tensor = input_noice_wav * scale_factor
    # 将缩放后的噪声张量和干净语音张量相加，实现添加噪声进行语音增强
    augmented_speech_tensor = input_clear_wav + scaled_noise_tensor
    torchaudio.save(f"./augmented_speech_snr_{snr}.wav", augmented_speech_tensor, sample_rate=16000)


def get_a_random_noise_snr():
    """
        信噪比具体情况如下：
    30： 隐约能听到背景噪音。
    15： 还是只能感受到最后的呲呲声音
    10： 开始能听到真正的敲门声音
    -30： 人话声有些小了
    -40： 隐约有点说话声音
    -45： 主要是噪音，基本没法听到人声
    训练的时候推荐从30到-30
    AI给出的比例：
    20-30： 10%  安静的图书馆或自习室 乡村户外安静的自然环境
    10-20: 35% 普通的室内办公室环境 街边的咖啡馆或餐厅
    0-10: 40%  热闹的商场或超市 正在行驶的公交车或地铁内
    -10 -0: 10% 嘈杂的工厂车间 喧闹的酒吧或夜总会
    -30 - -10: 5% 大型建筑工地  飞机起飞或降落的跑道附近
    :return:
    """
    # 根据上面的比例随机给出一个信噪比
    # 定义各个区间的边界和对应的概率
    intervals = [(20, 30), (10, 20), (0, 10), (-10, 0), (-30, -10)]
    probabilities = [0.1, 0.35, 0.4, 0.1, 0.05]
    # 根据概率从区间中选择一个区间
    selected_interval = random.choices(intervals, weights=probabilities)[0]
    # 在选定的区间内生成一个均匀分布的随机数
    random_number = random.uniform(selected_interval[0], selected_interval[1])
    return random_number


def show_musan_augment():
    input_clear_wav, _ = torchaudio.load(
        "/home/work_nfs15/asr_data/data/aishell_1/origin_wav/data_aishell/wav/test/S0764/BAC009S0764W0126.wav")
    input_noice_wav, _ = torchaudio.load("/home/work_nfs15/asr_data/data/musan/noise/noise-free-sound-0788.wav")
    utils_file.global_timer.start()
    do_noice_augment(input_clear_wav, input_noice_wav, 45)
    do_noice_augment(input_clear_wav, input_noice_wav, 40)
    do_noice_augment(input_clear_wav, input_noice_wav, 35)
    do_noice_augment(input_clear_wav, input_noice_wav, 30)  # 隐约能感觉到
    do_noice_augment(input_clear_wav, input_noice_wav, 25)
    utils_file.global_timer.stop_halfway_and_print()
    do_noice_augment(input_clear_wav, input_noice_wav, 20)
    utils_file.global_timer.stop_halfway_and_print()
    do_noice_augment(input_clear_wav, input_noice_wav, 15)  # 还是只能感受到最后的呲呲声音
    utils_file.global_timer.stop_halfway_and_print()
    do_noice_augment(input_clear_wav, input_noice_wav, 10)  # 开始能听到真正的敲门声音
    utils_file.global_timer.stop_halfway_and_print()
    do_noice_augment(input_clear_wav, input_noice_wav, 5)
    utils_file.global_timer.stop_halfway_and_print()
    do_noice_augment(input_clear_wav, input_noice_wav, 0)
    utils_file.global_timer.stop_halfway_and_print()
    do_noice_augment(input_clear_wav, input_noice_wav, -5)
    do_noice_augment(input_clear_wav, input_noice_wav, -10)
    do_noice_augment(input_clear_wav, input_noice_wav, -15)
    do_noice_augment(input_clear_wav, input_noice_wav, -20)
    do_noice_augment(input_clear_wav, input_noice_wav, -25)
    do_noice_augment(input_clear_wav, input_noice_wav, -30)  # 人话声有点小了
    do_noice_augment(input_clear_wav, input_noice_wav, -35)
    do_noice_augment(input_clear_wav, input_noice_wav, -40)  # 隐约有点说话声音
    do_noice_augment(input_clear_wav, input_noice_wav, -45)  # 主要是噪音，基本没法听到人声
    utils_file.global_timer.stop_halfway_and_print()

def show_random_srn():
    random.seed(10086)
    utils_file.global_timer.start()
    for i in range(3):
        print(get_a_random_noise_snr())
    utils_file.global_timer.stop_halfway_and_print()

# 下面考虑如何在wenet的dataloader中加载该augmentation
# 首先需要加载好big data_dict,包括音乐和噪声， 然后为了效率，每百条音频使用一个噪音（包含音乐）音频进行增强， 每个音频随机出一个信噪比数值。
class NoiceAugmentWorker:
    def __init__(self, noice_scp_or_list_path_list, replacement_frequency:int=100):
        self.noice_wav_list = []
        self.replacement_frequency = replacement_frequency
        for noice_scp_or_list_path in noice_scp_or_list_path_list:
            assert os.path.exists(noice_scp_or_list_path) and (noice_scp_or_list_path.endswith(".scp") or noice_scp_or_list_path.endswith(".list") or noice_scp_or_list_path.endswith(".txt"))
            if noice_scp_or_list_path.endswith(".scp"):
                tmp_wav_dict = utils_file.load_dict_from_scp(noice_scp_or_list_path)
                self.noice_wav_list.extend(list(tmp_wav_dict.values()))
            else:
                self.noice_wav_list.extend(utils_file.load_list_file_clean(noice_scp_or_list_path))
        self.num_count = 0 # 作为计数，如果计算到replacement_frequency，就更换当前noice wav
        self.now_noice_wav = torchaudio.load(random.choice(self.noice_wav_list))

    def do_apply_augment(self,input_clear_wav:torch.Tensor):
        """"""
        srn = self.get_a_random_noise_snr()
        self.do_noice_augment(input_clear_wav, self.now_noice_wav, srn)
        self.num_count += 1
        if self.num_count >= self.replacement_frequency:
            self.now_noice_wav = torchaudio.load(random.choice(self.noice_wav_list))
            self.num_count = 0

    @staticmethod
    def do_noice_augment(input_clear_wav: torch.Tensor, input_noice_wav: torch.Tensor, snr: float = 5):
        """
        信噪比具体情况如下：
        30： 隐约能听到背景噪音。
        15： 还是只能感受到最后的呲呲声音
        10： 开始能听到真正的敲门声音
        -30： 人话声有些小了
        -40： 隐约有点说话声音
        -45： 主要是噪音，基本没法听到人声
        训练的时候推荐从30到-30
        AI给出的比例：
        20-30： 10%  安静的图书馆或自习室 乡村户外安静的自然环境
        10-20: 35% 普通的室内办公室环境 街边的咖啡馆或餐厅
        0-10: 40%  热闹的商场或超市 正在行驶的公交车或地铁内
        -10 -0: 10% 嘈杂的工厂车间 喧闹的酒吧或夜总会
        -30 - -10: 5% 大型建筑工地  飞机起飞或降落的跑道附近

        :param input_clear_wav:(1,samples)
        :param input_noice_wav:(1,samples)
        :param snr: 信噪比
        :return:
        """
        while input_noice_wav.size(1) < input_clear_wav.size(1):
            input_noice_wav = torch.cat([input_noice_wav, input_noice_wav], dim=1)

        # 先计算语音的功率（这里明确在样本数量维度求平均来近似功率，保持维度方便后续计算）
        clean_power = torch.mean(input_clear_wav ** 2, dim=1, keepdim=True)
        # 计算噪声的功率，同样在对应维度求平均
        noise_power = torch.mean(input_noice_wav ** 2, dim=1, keepdim=True)

        # 根据信噪比公式推导出噪声需要缩放的因子
        scale_factor = torch.sqrt(clean_power / (noise_power * (10 ** (snr / 10))))
        # 先按照干净语音的长度裁剪噪声张量
        input_noice_wav = input_noice_wav[:, :input_clear_wav.shape[1]]
        # 对裁剪后的噪声张量进行缩放，以达到期望的信噪比
        scaled_noise_tensor = input_noice_wav * scale_factor
        # 将缩放后的噪声张量和干净语音张量相加，实现添加噪声进行语音增强
        augmented_speech_tensor = input_clear_wav + scaled_noise_tensor
        torchaudio.save(f"./augmented_speech_snr_{snr}.wav", augmented_speech_tensor, sample_rate=16000)

    @staticmethod
    def get_a_random_noise_snr():
        """
            信噪比具体情况如下：
        30： 隐约能听到背景噪音。
        15： 还是只能感受到最后的呲呲声音
        10： 开始能听到真正的敲门声音
        -30： 人话声有些小了
        -40： 隐约有点说话声音
        -45： 主要是噪音，基本没法听到人声
        训练的时候推荐从30到-30
        AI给出的比例：
        20-30： 10%  安静的图书馆或自习室 乡村户外安静的自然环境
        10-20: 35% 普通的室内办公室环境 街边的咖啡馆或餐厅
        0-10: 40%  热闹的商场或超市 正在行驶的公交车或地铁内
        -10 -0: 10% 嘈杂的工厂车间 喧闹的酒吧或夜总会
        -30 - -10: 5% 大型建筑工地  飞机起飞或降落的跑道附近
        :return:
        """
        # 根据上面的比例随机给出一个信噪比
        # 定义各个区间的边界和对应的概率
        intervals = [(20, 30), (10, 20), (0, 10), (-10, 0), (-30, -10)]
        probabilities = [0.1, 0.35, 0.4, 0.1, 0.05]
        # 根据概率从区间中选择一个区间
        selected_interval = random.choices(intervals, weights=probabilities)[0]
        # 在选定的区间内生成一个均匀分布的随机数
        random_number = random.uniform(selected_interval[0], selected_interval[1])
        return random_number

global_noise_augment_worker = NoiceAugmentWorker([
    '/home/work_nfs15/asr_data/data/musan/music/wav.scp',
    '/home/work_nfs15/asr_data/data/musan/noice/wav.scp',
], replacement_frequency=100)


if __name__ == '__main__':
    """"""
    show_random_srn()



from wenet.transformer.embedding import WhisperPositionalEncoding


class ParaformerPositinoalEncoding(WhisperPositionalEncoding):
    """ Sinusoids position encoding used in paraformer.encoder
    """

    def __init__(self,
                 depth: int,
                 d_model: int,
                 dropout_rate: float = 0.1,
                 max_len: int = 1500):
        super().__init__(depth, dropout_rate, max_len)
        self.xscale = d_model**0.5



import torch


def padding_mask(seq_q: torch.Tensor, seq_k: torch.Tensor, pad_id: int = 0):
    """
    根据seq_k 中pad_id的位置， 得到seq_q对于seq_k的padding_attn_mask,
    seq_q 只是提供q_len,并不影响mask的分布,
    input->
    seq_q: (batch_size, len_q)
    seq_k: (batch_size, len_k)
    output->
    mask: (batch_size, len_q, len_k)
    """
    batch_size, len_q = seq_q.size()
    batch_size, len_k = seq_k.size()
    mask = seq_k.eq(pad_id)
    mask = mask.unsqueeze(1).expand(batch_size, len_q, len_k)
    return mask





from gxl_ai_utils.utils import utils_file


def do_get_formatted_datalist_for_chat_task(input_wav, input_text, dataset_name):
    """
    这个函是为了给实验室理解大模型任务中要用到的数据生成标准格式的data.list
    标准格式:
      {
  "task": task_tag, # 如 "<TRANSCRIBE>"
  "key":  utt_id, # 如 "IC0001W0007"
  "wav": wav_path, # 如 "/home/backup_nfs/data-ASR/AIShell2/AISHELL-2/iOS/data/wav/C0001/IC0001W0007.wav"
  "txt": text, # 如 "天安门"
  "lang": language, # 如 "<CN>"
  "speaker": speaker_tag, # 如 "spk001"
  "emotion": emotion等分类标签 # 如 "<HAPPY>"
  "gender": 性别标签 # 如 "<MALE>"
  "extra":{class:"label", "duration": wav_length# 单位s，如2.05125，表示2.05秒}
  }

    :param input_text:
    :param input_wav:
    :return:
    """
    if not isinstance(input_wav, dict):
        assert isinstance(input_wav, str)
        input_wav = utils_file.load_dict_from_scp(input_wav)
    if not isinstance(input_text, dict):
        assert isinstance(input_text, str)
        input_text = utils_file.load_dict_from_scp(input_text)
    task_tag = "<S2TCHAT>"
    lang = "<CN>"
    speaker = "<NONE>"
    emotion  = "NEUTRAL"
    gender = "<NONE>"
    res_list = []
    for key, wav_path in utils_file.tqdm(input_wav.items(), desc="Generating formatted data.list for ASR task", total=len(input_wav)):
        # txt = input_text[key]  # 别忘了key值的对应性
        if key not in input_text:
            utils_file.logging_warning("Warning: {} not in input_text".format(key))
            continue
        txt = input_text[key]
        duration = 0
        try:
            duration = utils_file.do_get_wav_duration(wav_path)
        except:
            try:
                samples, rt = utils_file._get_sample_count_torchaudio(wav_path)
                duration = samples / rt
            except:
                # utils_file.logging_print('Error in getting duration of wav file: {}'.format(wav_path))
                duration = 0
        extra = {"duration": duration, "dataset": dataset_name}
        item_dict = {"task": task_tag, "key": key, "wav": wav_path,"txt": txt, "lang": lang, "speaker": speaker, "emotion": emotion, "gender": gender, "extra": extra}
        res_list.append(item_dict)
    return res_list


def do_get_formatted_datalist_for_asr_task(input_wav, input_text, dataset_name):
    """
    这个函是为了给实验室理解大模型任务中要用到的数据生成标准格式的data.list
    标准格式:
      {
  "task": task_tag, # 如 "<TRANSCRIBE>"
  "key":  utt_id, # 如 "IC0001W0007"
  "wav": wav_path, # 如 "/home/backup_nfs/data-ASR/AIShell2/AISHELL-2/iOS/data/wav/C0001/IC0001W0007.wav"
  "txt": text, # 如 "天安门"
  "lang": language, # 如 "<CN>"
  "speaker": speaker_tag, # 如 "spk001"
  "emotion": emotion等分类标签 # 如 "<HAPPY>"
  "gender": 性别标签 # 如 "<MALE>"
  "extra":{class:"label", "duration": wav_length# 单位s，如2.05125，表示2.05秒}
  }

    :param input_text:
    :param input_wav:
    :return:
    """
    if not isinstance(input_wav, dict):
        assert isinstance(input_wav, str)
        input_wav = utils_file.load_dict_from_scp(input_wav)
    if not isinstance(input_text, dict):
        assert isinstance(input_text, str)
        input_text = utils_file.load_dict_from_scp(input_text)
    task_tag = "<TRANSCRIBE>"
    lang = "<CN>"
    speaker = "<NONE>"
    emotion  = "NEUTRAL"
    gender = "<NONE>"
    res_list = []
    for key, wav_path in utils_file.tqdm(input_wav.items(), desc="Generating formatted data.list for ASR task", total=len(input_wav)):
        # txt = input_text[key]  # 别忘了key值的对应性
        if key not in input_text:
            utils_file.logging_warning("Warning: {} not in input_text".format(key))
            continue
        txt = input_text[key]
        duration = 0
        try:
            duration = utils_file.do_get_wav_duration(wav_path)
        except:
            try:
                samples, rt = utils_file._get_sample_count_torchaudio(wav_path)
                duration = samples / rt
            except:
                # utils_file.logging_print('Error in getting duration of wav file: {}'.format(wav_path))
                duration = 0
        extra = {"duration": duration, "dataset": dataset_name}
        item_dict = {"task": task_tag, "key": key, "wav": wav_path,"txt": txt, "lang": lang, "speaker": speaker, "emotion": emotion, "gender": gender, "extra": extra}
        res_list.append(item_dict)
    return res_list



input_origin_data_jsonl = "/home/work_nfs15/asr_data/data/chat_data/web_text_2019_text/train_20W_3.jsonl"
origin_wav_dir_path = "/home/work_nfs15/asr_data/data/chat_data/web_text_2019_text/train_20W_3_wav"
# 分布得到wav_dict 和 text_dict
wav_dict = utils_file.get_scp_for_wav_dir(origin_wav_dir_path,suffix=".wav")
# print(len(wav_dict))
# little_wav_dict = utils_file.get_subdict(wav_dict, 0, 10)
# utils_file.print_dict(little_wav_dict)
text_dict = {}
info_dict_list = utils_file.load_dict_list_from_jsonl(input_origin_data_jsonl)
for item in utils_file.tqdm(info_dict_list, desc="Generating text dict from jsonl"):
    key = str(item["key"])
    text_dict[key] = item['A']

#     if key in text_dict:
#         print('出现key值重复','老值：',text_dict[key],'新值：',f'{item["Q"]} {item["A"]}')
#     text_dict[item["key"]] = f'{item["Q"]} {item["A"]}'
# tittle_text_dict = {key: text_dict[key] for index, key in enumerate(text_dict) if index < 10}
# utils_file.print_dict(tittle_text_dict)

# check key 是否相同
# eq_num = 0
# neq_num = 0
# for key in wav_dict.keys():
#     if key in text_dict:
#         eq_num += 1
#     else:
#         neq_num +=1
# print("eq_num:",eq_num)
# print("neq_num" , neq_num)


# 得到data.list
data_list_path = "/home/work_nfs15/asr_data/data/chat_data/web_text_2019_text/common_format/train_20W_3/data.list"
# res_dict_list = do_get_formatted_datalist_for_chat_task(wav_dict, text_dict, "web_text_2019__text_train_20W_3")
# utils_file.write_dict_list_to_jsonl(res_dict_list, data_list_path)
shard_dir = "/home/node54_tmpdata/xlgeng/chat_data/web_text_2019_text/common_format/train_20W_3/shards"
from make_shard_for_common_format import make_shards
make_shards(data_list_path, shard_dir)





# Copyright    2022  Xiaomi Corp.        (authors: Fangjun Kuang)
#
# See ../LICENSE for clarification regarding multiple authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from collections import defaultdict
from typing import List, Optional, Tuple

from icefall.utils import is_module_available


class NgramLm:
    def __init__(
        self,
        fst_filename: str,
        backoff_id: int,
        is_binary: bool = False,
    ):
        """
        Args:
          fst_filename:
            Path to the FST.
          backoff_id:
            ID of the backoff symbol.
          is_binary:
            True if the given file is a binary FST.
        """
        if not is_module_available("kaldifst"):
            raise ValueError("Please 'pip install kaldifst' first.")

        import kaldifst

        if is_binary:
            lm = kaldifst.StdVectorFst.read(fst_filename)
        else:
            with open(fst_filename, "r") as f:
                lm = kaldifst.compile(f.read(), acceptor=False)

        if not lm.is_ilabel_sorted:
            kaldifst.arcsort(lm, sort_type="ilabel")

        self.lm = lm
        self.backoff_id = backoff_id

    def _process_backoff_arcs(
        self,
        state: int,
        cost: float,
    ) -> List[Tuple[int, float]]:
        """Similar to ProcessNonemitting() from Kaldi, this function
        returns the list of states reachable from the given state via
        backoff arcs.

        Args:
          state:
            The input state.
          cost:
            The cost of reaching the given state from the start state.
        Returns:
          Return a list, where each element contains a tuple with two entries:
            - next_state
            - cost of next_state
          If there is no backoff arc leaving the input state, then return
          an empty list.
        """
        ans = []

        next_state, next_cost = self._get_next_state_and_cost_without_backoff(
            state=state,
            label=self.backoff_id,
        )
        if next_state is None:
            return ans
        ans.append((next_state, next_cost + cost))
        ans += self._process_backoff_arcs(next_state, next_cost + cost)
        return ans

    def _get_next_state_and_cost_without_backoff(
        self, state: int, label: int
    ) -> Tuple[int, float]:
        """TODO: Add doc."""
        import kaldifst

        arc_iter = kaldifst.ArcIterator(self.lm, state)
        num_arcs = self.lm.num_arcs(state)

        # The LM is arc sorted by ilabel, so we use binary search below.
        left = 0
        right = num_arcs - 1
        while left <= right:
            mid = (left + right) // 2
            arc_iter.seek(mid)
            arc = arc_iter.value
            if arc.ilabel < label:
                left = mid + 1
            elif arc.ilabel > label:
                right = mid - 1
            else:
                return arc.nextstate, arc.weight.value

        return None, None

    def get_next_state_and_cost(
        self,
        state: int,
        label: int,
    ) -> Tuple[List[int], List[float]]:
        states = [state]
        costs = [0]

        extra_states_costs = self._process_backoff_arcs(
            state=state,
            cost=0,
        )

        for s, c in extra_states_costs:
            states.append(s)
            costs.append(c)

        next_states = []
        next_costs = []
        for s, c in zip(states, costs):
            ns, nc = self._get_next_state_and_cost_without_backoff(s, label)
            if ns:
                next_states.append(ns)
                next_costs.append(c + nc)

        return next_states, next_costs


class NgramLmStateCost:
    def __init__(self, ngram_lm: NgramLm, state_cost: Optional[dict] = None):
        assert ngram_lm.lm.start == 0, ngram_lm.lm.start
        self.ngram_lm = ngram_lm
        if state_cost is not None:
            self.state_cost = state_cost
        else:
            self.state_cost = defaultdict(lambda: float("inf"))

            # At the very beginning, we are at the start state with cost 0
            self.state_cost[0] = 0.0

    def forward_one_step(self, label: int) -> "NgramLmStateCost":
        state_cost = defaultdict(lambda: float("inf"))
        for s, c in self.state_cost.items():
            next_states, next_costs = self.ngram_lm.get_next_state_and_cost(
                s,
                label,
            )
            for ns, nc in zip(next_states, next_costs):
                state_cost[ns] = min(state_cost[ns], c + nc)

        return NgramLmStateCost(ngram_lm=self.ngram_lm, state_cost=state_cost)

    @property
    def lm_score(self) -> float:
        if len(self.state_cost) == 0:
            return float("-inf")

        return -1 * min(self.state_cost.values())



import logging
import platform
import os

BASE_PATH = os.path.dirname((os.path.abspath(__file__))) + "/"
# if platform.system().lower() == 'windows':
#     print("windows 环境")
#     BASE_PATH = "F:\code\python\deeplearning\pythonProject/gxl_ai_utils/"
# elif platform.system().lower() == 'linux':
#     print("linux 环境")
#     BASE_PATH = "/home/work_nfs7/xlgeng/gxl_dir1/gxl_ai_utils/gxl_ai_utils/"

DATA_PATH = BASE_PATH + "../gxl_data/"
OUTPUT_PATH = BASE_PATH + "../output/"
LOG_PATH = OUTPUT_PATH + "log/"


def AI_LOGGER(log_file_path=None):
    """
    将日志输出到日志文件和控制台
    """
    if log_file_path is None:
        no_file = True
        log_file_path = 'None'
    else:
        no_file = False
        os.makedirs(os.path.dirname(log_file_path), exist_ok=True)
    if log_file_path in logging.Logger.manager.loggerDict:
        return logging.getLogger(log_file_path)
    logger = logging.getLogger(log_file_path)
    logger.setLevel(logging.INFO)
    formatter = logging.Formatter(
        '%(asctime)s - %(levelname)s - %(message)s')

    # # 创建一个handler，用于将日志输出到控制台
    console = logging.StreamHandler()
    console.setLevel(logging.INFO)
    console.setFormatter(formatter)
    logger.addHandler(console)

    # 创建一个handler，用于写入日志文件
    if not no_file:
        file_handler = logging.FileHandler(
            filename=log_file_path, encoding='utf-8')
        file_handler.setFormatter(formatter)
        file_handler.setLevel(logging.INFO)
        logger.addHandler(file_handler)
    return logger



# from . import store_data
# from . import store_model
from . import AiConstant
# from . import run as run
from . import utils as utils
from . import config as config
from . import thread as thread
# from . import gxl_dataset_wenet as gxl_dataset
# from . import audio_model as audio_model
# from . import audio_dataset as audio_dataset
# from . import audio_handler as audio_handler
# from . import model_warehouse as model_warehouse


def hello_ai():
    """write by gengxuelong"""
    print('hello gxl_ai_utils, here is xuelong geng ')



import argparse
import os.path


from gxl_ai_utils.utils import utils_file
import re

def get_tag_from_str(input_str):
    # 使用正则表达式提取所有 <> 中的内容
    matches = re.findall(r'<.*?>', input_str)
    # 输出结果
    if len(matches)==0:
        return '<--no_tag-->'
    return matches[0].upper()


if __name__ == '__main__':
    """
    首先对识别结果进行计算，接着计算acc,
    分别存入wer文件和acc文件
    """
    parser = argparse.ArgumentParser()
    task = 'end_background'
    parser.add_argument('--ref_file', type=str, required=True, help='reference file, 真实的标签')
    parser.add_argument('--hyp_file', type=str, required=True, help='hypothesis file， 推理的标签')
    parser.add_argument('--output_dir', type=str, required=True, help='output dir 结果保存的目录')
    args = parser.parse_args()
    utils_file.makedir_sil(args.output_dir)
    utils_file.logging_info(f'开始计算{task}的wer')
    utils_file.do_compute_wer(args.ref_file, args.hyp_file, args.output_dir)
    utils_file.logging_info('字符错误率计算完毕，结果如下')
    res = utils_file.do_execute_shell_command(f'tail -n 8 {os.path.join(args.output_dir, "wer")}')
    print(res)

    utils_file.logging_info(f'开始计算{task}正确率，acc')
    acc_path = os.path.join(args.output_dir, 'acc')
    hyp_dict = utils_file.load_dict_from_scp(args.hyp_file)
    ref_dict = utils_file.load_dict_from_scp(args.ref_file)
    tag_hyp_dict = {}
    for key, value in hyp_dict.items():
        tag_hyp_dict[key] = get_tag_from_str(value)
    tag_ref_dict = {}
    for key, value in ref_dict.items():
        tag_ref_dict[key] = get_tag_from_str(value)
    output_acc_f = open(acc_path, 'w', encoding='utf-8')
    same_num = 0
    for key, hyp_tags in tag_hyp_dict.items():
        if key not in tag_ref_dict:
            continue
        ref_tags = tag_ref_dict[key]
        # 判断标签是否相同
        if ref_tags == hyp_tags:
            if_same = True
            same_num += 1
        else:
            if_same = False

        # 向文件写入数据
        output_acc_f.write(f"key: {key}\n")
        output_acc_f.write(f"ref_tag: {ref_tags}\n")
        output_acc_f.write(f"hyp_tag: {hyp_tags}\n")
        output_acc_f.write(f"if_same: {if_same}\n")
        output_acc_f.write("\n")  # 添加空行分隔不同的条目
    output_acc_f.write(f'正确率为：{same_num / len(tag_hyp_dict)}')
    # 输出相同标签的数量
    print(f"{task}的正确率为: {same_num / len(tag_hyp_dict)}")














import re

from tqdm import tqdm

from gxl_ai_utils.utils import utils_file

input_path = "/home/work_nfs8/xlgeng/new_workspace/checkpoint/download/xiaohuangji50w_nofenci.conv"


lines_all = utils_file.load_list_file_clean(input_path)
lines_num = len(lines_all)
print(lines_num)
index = 0
res_data = []
now_str = "Q: "
for line in tqdm(lines_all[:int(lines_num / 20)]):
    if line == "E":
        index += 1
        continue
    line = line.strip().replace("M ", "")
    if index == 1:
        now_str += line
        now_str += " A: "
        index +=1
        continue
    if index == 2:
        now_str += line
        res_data.append(now_str)
        index = 0
        now_str = "Q: "
        continue

utils_file.print_list(res_data[:10])


def get_QA(line):
    items = line.split("A: ")
    Q = items[0].replace("Q: ", "").replace(" ","").strip()
    A = items[1].replace(" ", "").strip()
    return Q, A

def has_letters(string):
    pattern = re.compile(r'[a-zA-Z]')
    return bool(pattern.search(string))


utils_file.logging_print("filtering")
res_data_new = []
for line in tqdm(res_data):
    Q, A = get_QA(line)
    Q = Q.strip()
    A = A.strip()
    if len(Q) > 30 or len(Q) < 7:
        continue
    if len(A) > 30 or len(A) < 7:
        continue
    if has_letters(Q) or has_letters(A):
        print(Q,A )
        continue
    lines = f'Q: {Q} A: {A}'
    res_data_new.append(lines)

utils_file.print_list(res_data_new[:10])
utils_file.logging_limit_print(len(res_data_new))

utils_file.write_list_to_file(res_data_new, "./xiaohuangji50w_nofenci.all")
dict_list = []
keys = []
for line in tqdm(res_data_new):
    Q, A = get_QA(line)
    key = utils_file.do_generate_random_num(20)
    while key in keys:
        key = utils_file.do_generate_random_num(20)
    keys.append(key)
    dict_i = {"key": key, "Q": Q, "A": A}
    dict_list.append(dict_i)
utils_file.write_dict_list_to_jsonl(dict_list, "./xiaohuangji50w_nofenci.josnl")





# Copyright (c) 2022  Mddct(hamddct@gmail.com)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import requests
import sys
import tarfile
from pathlib import Path
from urllib.request import urlretrieve

import tqdm


def download(url: str, dest: str, only_child=True):
    """ download from url to dest
    """
    assert os.path.exists(dest)
    print('Downloading {} to {}'.format(url, dest))

    def progress_hook(t):
        last_b = [0]

        def update_to(b=1, bsize=1, tsize=None):
            if tsize not in (None, -1):
                t.total = tsize
            displayed = t.update((b - last_b[0]) * bsize)
            last_b[0] = b
            return displayed

        return update_to

    # *.tar.gz
    name = url.split('?')[0].split('/')[-1]
    tar_path = os.path.join(dest, name)
    with tqdm.tqdm(unit='B',
                   unit_scale=True,
                   unit_divisor=1024,
                   miniters=1,
                   desc=(name)) as t:
        urlretrieve(url,
                    filename=tar_path,
                    reporthook=progress_hook(t),
                    data=None)
        t.total = t.n

    with tarfile.open(tar_path) as f:
        if not only_child:
            f.extractall(dest)
        else:
            for tarinfo in f:
                if "/" not in tarinfo.name:
                    continue
                name = os.path.basename(tarinfo.name)
                fileobj = f.extractfile(tarinfo)
                with open(os.path.join(dest, name), "wb") as writer:
                    writer.write(fileobj.read())


class Hub(object):
    """Hub for wenet pretrain runtime model
    """
    # TODO(Mddct): make assets class to support other language
    Assets = {
        # wenetspeech
        "chinese": "wenetspeech_u2pp_conformer_libtorch.tar.gz",
        # gigaspeech
        "english": "gigaspeech_u2pp_conformer_libtorch.tar.gz",
        # paraformer
        "paraformer": "paraformer.tar.gz"
    }

    def __init__(self) -> None:
        pass

    @staticmethod
    def get_model_by_lang(lang: str) -> str:
        if lang not in Hub.Assets.keys():
            print('ERROR: Unsupported language {} !!!'.format(lang))
            sys.exit(1)

        # NOTE(Mddct): model_dir structure
        # Path.Home()/.wenet
        # - chs
        #    - units.txt
        #    - final.zip
        # - en
        #    - units.txt
        #    - final.zip
        model = Hub.Assets[lang]
        model_dir = os.path.join(Path.home(), ".wenet", lang)
        if not os.path.exists(model_dir):
            os.makedirs(model_dir)
        # TODO(Mddct): model metadata
        if set(["final.zip",
                "units.txt"]).issubset(set(os.listdir(model_dir))):
            return model_dir
        # If not exist, download
        response = requests.get(
            "https://modelscope.cn/api/v1/datasets/wenet/wenet_pretrained_models/oss/tree"  # noqa
        )
        model_info = next(data for data in response.json()["Data"]
                          if data["Key"] == model)
        model_url = model_info['Url']
        download(model_url, model_dir, only_child=True)
        return model_dir



from gxl_ai_utils.utils import utils_file

def make_hlg():
    """"""
    input_words = '/home/work_nfs8/xlgeng/new_workspace/gxl_ai_utils/eggs/cats_and_dogs/prepare_data_for_en_cn/make_arpa/data_res/words_lines.txt'
    input_tokens = '/home/work_nfs7/yhliang/online_sys/wenet/examples/onlinesys/data/dict/lang_char.txt'
    arpa_path = "/home/work_nfs8/xlgeng/new_workspace/gxl_ai_utils/eggs/cats_and_dogs/prepare_data_for_en_cn/make_arpa/data_res/output.arpa"
    output_dir = './data_res'
    utils_file.makedir_sil(output_dir)
    utils_file.do_make_HLG(input_words, input_tokens, arpa_path, output_dir)

if __name__ == '__main__':
    make_hlg()


import ast
import random
from gxl_ai_utils.utils import utils_file
from matplotlib import pyplot as plt

from Gxl2DTable import Gxl2DTable
if __name__ == '__main__':
    plt.rcParams.update({'font.size': 15})
    wer_info_table = Gxl2DTable.load_from_xlsx(
        "../xlsx_data/down_2_stage_3.xlsx",
        return_cols=True)
    data_dict = wer_info_table.dict_info
    common_list = data_dict["common"]
    short_domain_list = data_dict["short"]
    long_domain_list = data_dict["long"]
    short_list = [ (common_list[i]-item) / common_list[i]*100 for i, item in enumerate(short_domain_list)]
    long_list = [ (common_list[i]-item) / common_list[i]*100 for i, item in enumerate(long_domain_list)]
    common_messy_list = [ (common_list[i]-item) / common_list[i]*100 for i, item in enumerate(common_list)]
    # X轴的值
    x_values = list(range(27))

    # 创建折线图
    plt.figure(figsize=(10, 6))
    plt.plot(x_values, long_list, label='long-domain', marker='o')
    plt.plot(x_values, short_list, label='short-domain', marker='s')
    plt.plot(x_values, common_messy_list, label='common', marker='^')

    # 添加标题和标签
    # plt.title('Performance improvement percentage of domain prompts compared to common prompt.')
    plt.xlabel('Index (speechio_*)')
    plt.ylabel('Value (%)')

    # 设置X轴刻度
    plt.xticks(x_values)
    plt.xticks(rotation=45)
    # 设置X轴刻度
    plt.xticks(x_values)
    # 添加图例
    plt.legend()
    plt.subplots_adjust(left=0.10, right=0.98, bottom=0.13, top=0.98)  # 调整左右边距
    # 显示网格
    plt.grid()
    # plt.show()
    # 保存图形
    plt.savefig('./res/long_short_domain.pdf', bbox_inches='tight')


def amin():
    plt.rcParams.update({'font.size': 15})
    info_dict = utils_file.load_data_from_xlsx("../tongyi_fenxi/down2_stage3__version2/data/wer_paths2.xlsx", return_cols=False)
    # for common
    common_list = []
    type_index = 1# 2 delete, 3 insert
    for i in range(27):
        key = f"speechio_{i}"
        common_wer_path = info_dict[key][0]
        print(common_wer_path)
        res_wer = utils_file.get_wer_all_from_wer_file(common_wer_path)
        print(res_wer)
        replace_num = int(res_wer[type_index])
        common_list.append(replace_num)
    short_list = []
    for i in range(27):
        key = f"speechio_{i}"
        common_wer_path = info_dict[key][1]
        print(common_wer_path)
        res_wer = utils_file.get_wer_all_from_wer_file(common_wer_path)
        print(res_wer)
        replace_num = int(res_wer[type_index])# 2: delete 3: insert
        short_list.append(replace_num)
    long_list = []
    for i in range(27):
        key = f"speechio_{i}"
        common_wer_path = info_dict[key][2]
        print(common_wer_path)
        res_wer = utils_file.get_wer_all_from_wer_file(common_wer_path)
        print(res_wer)
        replace_num = int(res_wer[type_index])
        long_list.append(replace_num)

    # 得到提升的百分比
    messy_percent_list = [(common_list[i] - item) / common_list[i] * 100 for i, item in enumerate(long_list)]
    short_messy_list = [(common_list[i] - item) / common_list[i] * 100 for i, item in enumerate(short_list)]
    common_messy_list = [(common_list[i] - item) / common_list[i] * 100 for i, item in enumerate(common_list)]
    # X轴的值
    x_values = list(range(27))

    # 创建折线图
    plt.figure(figsize=(10, 6))
    plt.plot(x_values, messy_percent_list, label='long-domain', marker='o')
    plt.plot(x_values, short_messy_list, label='short-domain', marker='s')
    plt.plot(x_values, common_messy_list, label='common', marker='^')

    # 添加标题和标签
    # plt.title('Performance improvement percentage of domain prompts compared to common prompt in substitution errors.')
    plt.xlabel('Index (speechio_*)')
    plt.ylabel('Value (%)')

    # 设置X轴刻度
    plt.xticks(x_values)
    plt.xticks(rotation=45)
    # 设置X轴刻度
    plt.xticks(x_values)
    # 添加图例
    plt.legend()
    plt.subplots_adjust(left=0.10, right=0.98, bottom=0.13, top=0.98)  # 调整左右边距
    # 显示网格
    plt.grid()
    # plt.show()
    # 保存图形
    plt.savefig('./res/long_short_domain_replace_errs.pdf', bbox_inches='tight')


import os

from make_shard_common import make_shards_common
from make_datalist4tokendata import do_get_formatted_datalist_for_token_task
from gxl_ai_utils.utils import utils_file

token_path = "/mnt/sfs/asr/code/gxl_ai_utils/eggs/cat_and_dogs_stage2/cosyvoice1_text2token/exp/output/attention_rescoring/0.pt/text_aishell_chat_znlin.scp"
wav_path = "/mnt/sfs/asr/update_data/raw_data/asr_chat_znlin_2025-1-24/wav.scp"
text_path = "/mnt/sfs/asr/update_data/raw_data/asr_chat_znlin_2025-1-24/text.scp"
output_path = "/mnt/sfs/asr/update_data/speech2text_token/asr_chat_znlin_enhance_2025-3-5"
utils_file.makedir_sil(output_path)
dict_list = do_get_formatted_datalist_for_token_task(
    wav_path,
    text_path,
    token_path,
    "speech2text_asr_chat_znlin_enhance_2025-3-5"
)
data_list_path = os.path.join(output_path, "data.list")
utils_file.write_dict_list_to_jsonl(
    dict_list,
    data_list_path
)
shards_dir = os.path.join(output_path, "shards")
utils_file.makedir_sil(shards_dir)
make_shards_common(
    data_list_path,
    shards_dir,
    num_threads=32
)


# Copyright (c)  2021  Xiaomi Corporation (authors: Fangjun Kuang)
#
# See ../../../../LICENSE for clarification regarding multiple authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
from typing import Tuple

import torch
import torch.nn.functional as F

from icefall.utils import add_eos, add_sos, make_pad_mask


class RnnLmModel(torch.nn.Module):
    def __init__(
        self,
        vocab_size: int,
        embedding_dim: int,
        hidden_dim: int,
        num_layers: int,
        tie_weights: bool = False,
    ):
        """
        Args:
          vocab_size:
            Vocabulary size of BPE model.
          embedding_dim:
            Input embedding dimension.
          hidden_dim:
            Hidden dimension of RNN layers.
          num_layers:
            Number of RNN layers.
          tie_weights:
            True to share the weights between the input embedding layer and the
            last output linear layer. See https://arxiv.org/abs/1608.05859
            and https://arxiv.org/abs/1611.01462
        """
        super().__init__()
        self.vocab_size = vocab_size
        self.embedding_dim = embedding_dim
        self.hidden_dim = hidden_dim
        self.num_layers = num_layers
        self.tie_weights = tie_weights

        self.input_embedding = torch.nn.Embedding(
            num_embeddings=vocab_size,
            embedding_dim=embedding_dim,
        )

        self.rnn = torch.nn.LSTM(
            input_size=embedding_dim,
            hidden_size=hidden_dim,
            num_layers=num_layers,
            batch_first=True,
        )

        self.output_linear = torch.nn.Linear(
            in_features=hidden_dim, out_features=vocab_size
        )

        self.vocab_size = vocab_size
        if tie_weights:
            logging.info("Tying weights")
            assert embedding_dim == hidden_dim, (embedding_dim, hidden_dim)
            self.output_linear.weight = self.input_embedding.weight
        else:
            logging.info("Not tying weights")

        self.cache = {}

    def streaming_forward(
        self,
        x: torch.Tensor,
        y: torch.Tensor,
        h0: torch.Tensor,
        c0: torch.Tensor,
    ) -> Tuple[torch.Tensor, torch.Tensor, torch.Tensor]:
        """
        Args:
          x:
            A 2-D tensor of shape (N, L). We won't prepend it with SOS.
          y:
            A 2-D tensor of shape (N, L). We won't append it with EOS.
          h0:
            A 3-D tensor of shape (num_layers, N, hidden_size).
            (If proj_size > 0, then it is (num_layers, N, proj_size))
          c0:
            A 3-D tensor of shape (num_layers, N, hidden_size).
        Returns:
          Return a tuple containing 3 tensors:
            - negative loglike (nll), a 1-D tensor of shape (N,)
            - next_h0, a 3-D tensor with the same shape as h0
            - next_c0, a 3-D tensor with the same shape as c0
        """
        assert x.ndim == y.ndim == 2, (x.ndim, y.ndim)
        assert x.shape == y.shape, (x.shape, y.shape)

        # embedding is of shape (N, L, embedding_dim)
        embedding = self.input_embedding(x)
        # Note: We use batch_first==True
        rnn_out, (next_h0, next_c0) = self.rnn(embedding, (h0, c0))
        logits = self.output_linear(rnn_out)
        nll_loss = F.cross_entropy(
            logits.reshape(-1, self.vocab_size), y.reshape(-1), reduction="none"
        )

        batch_size = x.size(0)
        nll_loss = nll_loss.reshape(batch_size, -1).sum(dim=1)
        return nll_loss, next_h0, next_c0

    def forward(
        self, x: torch.Tensor, y: torch.Tensor, lengths: torch.Tensor
    ) -> torch.Tensor:
        """
        Args:
          x:
            A 2-D tensor with shape (N, L). Each row
            contains token IDs for a sentence and starts with the SOS token.
          y:
            A shifted version of `x` and with EOS appended.
          lengths:
            A 1-D tensor of shape (N,). It contains the sentence lengths
            before padding.
        Returns:
          Return a 2-D tensor of shape (N, L) containing negative log-likelihood
          loss values. Note: Loss values for padding positions are set to 0.
        """
        assert x.ndim == y.ndim == 2, (x.ndim, y.ndim)
        assert lengths.ndim == 1, lengths.ndim
        assert x.shape == y.shape, (x.shape, y.shape)

        batch_size = x.size(0)
        assert lengths.size(0) == batch_size, (lengths.size(0), batch_size)

        # embedding is of shape (N, L, embedding_dim)
        embedding = self.input_embedding(x)

        # Note: We use batch_first==True
        rnn_out, _ = self.rnn(embedding)
        logits = self.output_linear(rnn_out)

        # Note: No need to use `log_softmax()` here
        # since F.cross_entropy() expects unnormalized probabilities

        # nll_loss is of shape (N*L,)
        # nll -> negative log-likelihood
        nll_loss = F.cross_entropy(
            logits.reshape(-1, self.vocab_size), y.reshape(-1), reduction="none"
        )
        # Set loss values for padding positions to 0
        mask = make_pad_mask(lengths).reshape(-1)
        nll_loss.masked_fill_(mask, 0)

        nll_loss = nll_loss.reshape(batch_size, -1)

        return nll_loss

    def predict_batch(self, tokens, token_lens, sos_id, eos_id, blank_id):
        device = next(self.parameters()).device
        batch_size = len(token_lens)

        sos_tokens = add_sos(tokens, sos_id)
        tokens_eos = add_eos(tokens, eos_id)
        sos_tokens_row_splits = sos_tokens.shape.row_splits(1)

        sentence_lengths = sos_tokens_row_splits[1:] - sos_tokens_row_splits[:-1]

        x_tokens = sos_tokens.pad(mode="constant", padding_value=blank_id)
        y_tokens = tokens_eos.pad(mode="constant", padding_value=blank_id)

        x_tokens = x_tokens.to(torch.int64).to(device)
        y_tokens = y_tokens.to(torch.int64).to(device)
        sentence_lengths = sentence_lengths.to(torch.int64).to(device)

        embedding = self.input_embedding(x_tokens)

        # Note: We use batch_first==True
        rnn_out, states = self.rnn(embedding)
        logits = self.output_linear(rnn_out)
        mask = torch.zeros(logits.shape).bool().to(device)
        for i in range(batch_size):
            mask[i, token_lens[i], :] = True
        logits = logits[mask].reshape(batch_size, -1)

        return logits[:, :].log_softmax(-1), states

    def clean_cache(self):
        self.cache = {}

    def score_token(self, x: torch.Tensor, x_lens: torch.Tensor, state=None):
        """Score a batch of tokens, i.e each sample in the batch should be a
        single token. For example, x = torch.tensor([[5],[10],[20]])


        Args:
            x (torch.Tensor):
                A batch of tokens
            x_lens (torch.Tensor):
                The length of tokens in the batch before padding
            state (optional):
                Either None or a tuple of two torch.Tensor. Each tensor has
                the shape of (num_layers, bs, hidden_dim)

        Returns:
            _type_: _description_
        """
        device = next(self.parameters()).device
        batch_size = x.size(0)
        if state:
            h, c = state
        else:
            h = torch.zeros(self.rnn.num_layers, batch_size, self.rnn.hidden_size).to(
                device
            )
            c = torch.zeros(self.rnn.num_layers, batch_size, self.rnn.hidden_size).to(
                device
            )

        embedding = self.input_embedding(x)
        rnn_out, states = self.rnn(embedding, (h, c))
        logits = self.output_linear(rnn_out)

        return logits[:, 0].log_softmax(-1), states

    def score_token_onnx(
        self,
        x: torch.Tensor,
        state_h: torch.Tensor,
        state_c: torch.Tensor,
    ) -> Tuple[torch.Tensor, torch.Tensor, torch.Tensor]:
        """Score a batch of tokens, i.e each sample in the batch should be a
        single token. For example, x = torch.tensor([[5],[10],[20]])


        Args:
            x (torch.Tensor):
                A batch of tokens
            state_h:
                state h of RNN has the shape of (num_layers, bs, hidden_dim)
            state_c:
                state c of RNN has the shape of (num_layers, bs, hidden_dim)

        Returns:
            _type_: _description_
        """
        embedding = self.input_embedding(x)
        rnn_out, (next_h0, next_c0) = self.rnn(embedding, (state_h, state_c))
        logits = self.output_linear(rnn_out)

        return logits[:, 0].log_softmax(-1), next_h0, next_c0

    def forward_with_state(
        self, tokens, token_lens, sos_id, eos_id, blank_id, state=None
    ):
        batch_size = len(token_lens)
        if state:
            h, c = state
        else:
            h = torch.zeros(self.rnn.num_layers, batch_size, self.rnn.hidden_size)
            c = torch.zeros(self.rnn.num_layers, batch_size, self.rnn.hidden_size)

        device = next(self.parameters()).device

        sos_tokens = add_sos(tokens, sos_id)
        tokens_eos = add_eos(tokens, eos_id)
        sos_tokens_row_splits = sos_tokens.shape.row_splits(1)

        sentence_lengths = sos_tokens_row_splits[1:] - sos_tokens_row_splits[:-1]

        x_tokens = sos_tokens.pad(mode="constant", padding_value=blank_id)
        y_tokens = tokens_eos.pad(mode="constant", padding_value=blank_id)

        x_tokens = x_tokens.to(torch.int64).to(device)
        y_tokens = y_tokens.to(torch.int64).to(device)
        sentence_lengths = sentence_lengths.to(torch.int64).to(device)

        embedding = self.input_embedding(x_tokens)

        # Note: We use batch_first==True
        rnn_out, states = self.rnn(embedding, (h, c))
        logits = self.output_linear(rnn_out)

        return logits, states



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright [2023-11-28] <sxc19@mails.tsinghua.edu.cn, Xingchen Song>
import torch
from torch.nn import BatchNorm1d, LayerNorm
from wenet.paraformer.embedding import ParaformerPositinoalEncoding
from wenet.transformer.norm import RMSNorm
from wenet.transformer.positionwise_feed_forward import (
    GatedVariantsMLP, MoEFFNLayer, PositionwiseFeedForward)

from wenet.transformer.swish import Swish
from wenet.transformer.subsampling import (
    LinearNoSubsampling,
    EmbedinigNoSubsampling,
    Conv1dSubsampling2,
    Conv2dSubsampling4,
    Conv2dSubsampling6,
    Conv2dSubsampling8,
    StackNFramesSubsampling,
)
from wenet.efficient_conformer.subsampling import Conv2dSubsampling2
from wenet.squeezeformer.subsampling import DepthwiseConv2dSubsampling4
from wenet.transformer.embedding import (PositionalEncoding,
                                         RelPositionalEncoding,
                                         WhisperPositionalEncoding,
                                         LearnablePositionalEncoding,
                                         NoPositionalEncoding)
from wenet.transformer.attention import (MultiHeadedAttention,
                                         MultiHeadedCrossAttention,
                                         RelPositionMultiHeadedAttention,
                                         ShawRelPositionMultiHeadedAttention)
from wenet.efficient_conformer.attention import (
    GroupedRelPositionMultiHeadedAttention)

WENET_ACTIVATION_CLASSES = {
    "hardtanh": torch.nn.Hardtanh,
    "tanh": torch.nn.Tanh,
    "relu": torch.nn.ReLU,
    "selu": torch.nn.SELU,
    "swish": getattr(torch.nn, "SiLU", Swish),
    "gelu": torch.nn.GELU,
}

WENET_RNN_CLASSES = {
    "rnn": torch.nn.RNN,
    "lstm": torch.nn.LSTM,
    "gru": torch.nn.GRU,
}

WENET_SUBSAMPLE_CLASSES = {
    "linear": LinearNoSubsampling,
    "embed": EmbedinigNoSubsampling,
    "conv1d2": Conv1dSubsampling2,
    "conv2d2": Conv2dSubsampling2,
    "conv2d": Conv2dSubsampling4,
    "dwconv2d4": DepthwiseConv2dSubsampling4,
    "conv2d6": Conv2dSubsampling6,
    "conv2d8": Conv2dSubsampling8,
    'paraformer_dummy': torch.nn.Identity,
    'stack_n_frames': StackNFramesSubsampling,
}

WENET_EMB_CLASSES = {
    "embed": PositionalEncoding,
    "abs_pos": PositionalEncoding,
    "rel_pos": RelPositionalEncoding,
    "no_pos": NoPositionalEncoding,
    "abs_pos_whisper": WhisperPositionalEncoding,
    "embed_learnable_pe": LearnablePositionalEncoding,
    "abs_pos_paraformer": ParaformerPositinoalEncoding,
}

WENET_ATTENTION_CLASSES = {
    "selfattn": MultiHeadedAttention,
    "rel_selfattn": RelPositionMultiHeadedAttention,
    "grouped_rel_selfattn": GroupedRelPositionMultiHeadedAttention,
    "crossattn": MultiHeadedCrossAttention,
    'shaw_rel_selfattn': ShawRelPositionMultiHeadedAttention
}

WENET_MLP_CLASSES = {
    'position_wise_feed_forward': PositionwiseFeedForward,
    'moe': MoEFFNLayer,
    'gated': GatedVariantsMLP
}

WENET_NORM_CLASSES = {
    'layer_norm': LayerNorm,
    'batch_norm': BatchNorm1d,
    'rms_norm': RMSNorm
}



import sys
sys.path.insert(0, "../../../")
from gxl_ai_utils.utils import utils_file

# root_output_dir = "/mnt/sfs/asr/update_data/asr2chat_by_llm"
#
# # 得到wenetspeech的所有shards路径
# all_shards_list_path = "/mnt/sfs/asr/asr/shards_list.txt"
# lines = utils_file.load_list_file_clean(all_shards_list_path)
# res_list = []
# for line_i in lines:
#     if "wenetspeech" in line_i:
#         res_list.append(line_i)
# output_wenetspeech_shards_list_path = root_output_dir + "/wenetspeech_shards_list.txt"
# utils_file.write_list_to_file(res_list, output_wenetspeech_shards_list_path)
#
# # 开始解压全部结果
# wav_path = f'{root_output_dir}/wav.scp'
# text_path = f'{root_output_dir}/text'
# # 对虽有wenetspeech进行tar包解压
# utils_file.do_uncompress_shard(output_wenetspeech_shards_list_path, f'{root_output_dir}/origin_wav',
#                                num_thread=32,
#                                wav_path=wav_path,
#                                text_path=text_path)

# 通过raw file得到common format data.list
wav_dir = "/home/work_nfs15/asr_data/data/wenetspeech/train"
wav_path = "/home/work_nfs15/asr_data/data/wenetspeech/train/wav.scp"
# utils_file.do_get_scp_for_wav_dir(wav_dir, wav_path, recursive=True)

text_path = "/home/work_nfs15/asr_data/data/wenetspeech/train_shards/wenetspeech_new_all/text"

fake_wav_path = utils_file.do_get_fake_file()
# wav_dict = utils_file.load_dict_from_scp(wav_path)
new_txt_dict_path = "/home/work_nfs15/asr_data/data/wenetspeech/wenetspeech_asr_chat_by_llm/gxl_1800.list"
new_txt_path = "/home/work_nfs15/asr_data/data/wenetspeech/wenetspeech_asr_chat_by_llm/text"
new_wav_path = "/home/work_nfs15/asr_data/data/wenetspeech/wenetspeech_asr_chat_by_llm/wav.scp"
# new_text_dict = utils_file.load_dict_from_scp(new_txt_dict_path)
# wav_dict_2 = {}
# text_dict_2 = {}
# for key, value in new_text_dict.items():
#     new_key = key+"_wenet"
#     if new_key not in wav_dict or new_key not in wav_dict:
#         print(f"{new_key} not in wav_dict or text_dict")
#         continue
#     wav_path_i = wav_dict[new_key]
#     text_items = value.strip().split()
#     if len(text_items) != 2:
#         print(f"{key} text_items not 2, value : {value}, len: {len(text_items)}")
#         continue
#     new_text = "<开始回答>".join(text_items)
#     text_dict_2[new_key] = new_text
#     wav_dict_2[new_key] = wav_path_i
#
# utils_file.write_dict_to_scp(text_dict_2, new_txt_path)
# utils_file.write_dict_to_scp(wav_dict_2, new_wav_path)


# 对data.list 进行压缩
# data_list = utils_file.do_get_formatted_datalist_for_all_task(
#     new_wav_path,
#     new_txt_path,
#     dataset_name='wenetspeech_by_llm_asr_chat',
#     task_tag="<TRANSCRIBE> <S2TCHAT>"
# )
output_raw_path =  "/home/work_nfs15/asr_data/data/wenetspeech/wenetspeech_asr_chat_by_llm/data.list"
# utils_file.write_dict_list_to_jsonl(data_list, output_raw_path)

from make_shard_for_common_format import make_shards
make_shards(
    output_raw_path,
    "/home/work_nfs15/asr_data/data/wenetspeech/wenetspeech_asr_chat_by_llm/shards",
    num_utts_per_shard=1000,
    prefix='wenetspeech_by_llm_asr_chat',
    resample=16000,
    num_threads=32
)


import os

from gxl_ai_utils.utils import utils_file

def data_handler():
    """"""
    input_dir = "/home/work_nfs14/xlgeng/asr_data_shard/wenetspeech_new_all"
    output_dir = "/home/work_nfs8/xlgeng/new_workspace/icefall/egs/multi_zh_en/ASR/gxl_data"
    wav_path = os.path.join(input_dir, "wav.scp")
    text_path = os.path.join(input_dir, "text")
    wav_file_list = utils_file.load_list_file_clean(wav_path)
    wav_file_list_list = utils_file.do_split_list(wav_file_list, 2)
    text_all_dict = utils_file.load_dict_from_scp(text_path)
    for i in range(len(wav_file_list_list)):
        output_dir_i = os.path.join(output_dir, f'wenetspeech_{i}')
        os.makedirs(output_dir_i, exist_ok=True)
        utils_file.write_list_to_file(wav_file_list_list[i], os.path.join(output_dir_i, "wav.scp"))
        temp_wav_dict = utils_file.load_dict_from_scp(os.path.join(output_dir_i, "wav.scp"))
        temp_text_dict = {k:text_all_dict[k] for k in temp_wav_dict.keys()}
        utils_file.write_dict_to_scp(temp_text_dict, os.path.join(output_dir_i, "text"))

def data_handler2():
    """"""
    input_dir = "/home/work_nfs8/xlgeng/new_workspace/icefall/egs/multi_zh_en/ASR/gxl_data"
    input_dir_i = os.path.join(input_dir, f'wenetspeech_{0}')
    wav_path = os.path.join(input_dir_i, "wav.scp")
    text_path = os.path.join(input_dir_i, "text")
    # utils_file.do_make_data4icefall(wav_path, text_path, parent_dir=input_dir_i)
    manifest_dir = os.path.join(input_dir_i, 'manifest')
    fbank_dir = os.path.join(input_dir_i, 'fbank_common')
    utils_file.makedir_sil(manifest_dir)
    utils_file.makedir_sil(fbank_dir)
    utils_file._do_compute_fbank4icefall(
            manifests_dir=manifest_dir,
            fbank_dir=fbank_dir,
            partition='train',
            prefix='wenetspeech_0',
            perturb_speed=True
        )

import torch
if __name__ == '__main__':
    data_handler()


import random

import numpy as np
import pyroomacoustics as pra
import tqdm
from scipy.io import wavfile
from scipy.signal import fftconvolve
from gxl_ai_utils.utils import utils_file


def get_rir():
    # 创建一个房间
    room_x = round(random.uniform(5, 8), 1)
    room_y = round(random.uniform(3, 5), 1)
    room_z = round(random.uniform(3, 4), 1)
    room_dim = [room_x, room_y, room_z]

    rt60_tgt = round(random.uniform(0.2, 1.2), 2)
    e_absorption, max_order = pra.inverse_sabine(rt60_tgt, room_dim)  # 0.1614901799818631 95
    print(e_absorption, max_order)
    room_dim = [10, 10, 3]  # 房间的尺寸
    mic_array_loc = [5, 5, 1]  # 麦克风阵列的位置
    source_loc = [3, 3, 1]  # 声源的位置
    room = pra.ShoeBox(room_dim, fs=16000, max_order=max_order, absorption=e_absorption)

    # 添加麦克风阵列
    mic_array = pra.MicrophoneArray(np.array([mic_array_loc]).T, room.fs)
    room.add_microphone_array(mic_array)

    # 添加声源
    room.add_source(source_loc)

    # 计算RIR
    room.compute_rir()
    rir = room.rir[0][0]
    return rir

EPS = np.finfo(float).eps
def normalize(audio, target_level=-25):
    '''Normalize the signal to the target level'''
    rms = (audio ** 2).mean() ** 0.5
    scalar = 10 ** (target_level / 20) / (rms + EPS)
    audio = audio * scalar
    return audio


def write_wav_with_normal(output_path, audio_reverb, fs):
    level_db = random.uniform(-45, -15)
    audio = normalize(audio_reverb.astype(np.int16), -15)
    # 保存带有混响的音频
    wavfile.write(output_path, fs, audio)


def apply_rir_to_wav(wav_dict, output_dir, rir):
    # import pdb;pdb.set_trace()
    utils_file.makedir_sil(output_dir)
    # 音频文件列表
    # wav_dict = utils_file.load_dict_from_scp(wav_scp_path)
    for key, audio_path in tqdm.tqdm(wav_dict.items(), desc="apply_rir_to_wav", total=len(wav_dict)):
        # 读取音频文件
        fs, audio = wavfile.read(audio_path)
        output_path = utils_file.do_replace_dir(audio_path, output_dir)
        # 将RIR应用到音频上
        audio_reverb = fftconvolve(audio, rir)[:len(audio)]
        write_wav_with_normal(output_path, audio_reverb, fs)

def main(input_scp_path, output_dir):
    now = utils_file.do_get_now_time()
    rir = get_rir()
    utils_file.logging_print('get rir耗时: %s' % (utils_file.do_get_now_time() - now))
    thread_num = 32
    wav_dict = utils_file.load_dict_from_scp(input_scp_path)
    wav_dict_list = utils_file.do_split_dict(wav_dict, thread_num)
    runner = utils_file.GxlFixedThreadPool(thread_num)
    now = utils_file.do_get_now_time()
    for wav_dict_i in wav_dict_list:
        runner.add_thread(apply_rir_to_wav, [wav_dict_i, output_dir, rir])
    runner.start()
    utils_file.logging_print('耗时: %s' % (utils_file.do_get_now_time() - now))


import numpy as np
import soundfile as sf


def read_audio(filename):
    audio, sr = sf.read(filename)
    if sr != 16000:
        audio = sf.resample(audio, 16000)
    if audio.ndim > 1:
        audio = np.mean(audio, axis=1)  # 合并多个通道
        print('多通道')
    audio = audio.flatten()
    return audio


if __name__ == "__main__":
    #  (72核)CPU
    # 20进程:   4.918259620666504/7176个
    # 32进程:    4.5718605518341064/7176个
    # 40进程  4.63465428352356/7176个
    # 72进程  5.503162145614624/7176个
    # 140进程   6.536927223205566/7176个
    # input_scp_path= "/home/work_nfs8/xlgeng/data/scp_test/aishell/wav.scp"
    # output_dir = './wav_output/aishell_test2'
    # main(input_scp_path, output_dir)
    rir = get_rir()
    print(rir)
    print(rir.shape)
    print(type(rir))
    print(rir.dtype)
    # rir_my = read_audio(
    #     "/Users/xuelonggeng/Desktop/xlgeng_workspace/gxl_ai_utils/eggs/cats_and_dogs/rir_shujuzengguang/other/simulated_rirs_16k/largeroom/Room001/Room001-00001.wav")
    # print(rir_my)
    # print(rir_my.shape)
    # print(type(rir_my))
    # print(rir_my.dtype)
    thread_num = 2
    input_scp_path = "/Users/xuelonggeng/Desktop/xlgeng_workspace/gxl_ai_utils/eggs/cats_and_dogs/rir_shujuzengguang/data/scp/wav.scp"
    output_dir = 'data/wav_output/aishell_test_my'
    wav_dict = utils_file.load_dict_from_scp(input_scp_path)
    wav_dict_list = utils_file.do_split_dict(wav_dict, thread_num)
    runner = utils_file.GxlFixedThreadPool(thread_num)
    now = utils_file.do_get_now_time()
    for wav_dict_i in wav_dict_list:
        runner.add_thread(apply_rir_to_wav, [wav_dict_i, output_dir, rir])
    runner.start()
    utils_file.logging_print('耗时: %s' % (utils_file.do_get_now_time() - now))

    output_dir = 'data/wav_output/aishell_test_gen'
    wav_dict = utils_file.load_dict_from_scp(input_scp_path)
    wav_dict_list = utils_file.do_split_dict(wav_dict, thread_num)
    runner = utils_file.GxlFixedThreadPool(thread_num)
    now = utils_file.do_get_now_time()
    for wav_dict_i in wav_dict_list:
        runner.add_thread(apply_rir_to_wav, [wav_dict_i, output_dir, rir])
    runner.start()
    utils_file.logging_print('耗时: %s' % (utils_file.do_get_now_time() - now))



# Copyright      2021  Xiaomi Corp.        (authors: Fangjun Kuang)
#
# See ../../LICENSE for clarification regarding multiple authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import os

import torch
from torch import distributed as dist


def setup_dist(
    rank=None, world_size=None, master_port=None, use_ddp_launch=False, master_addr=None
):
    """
    rank and world_size are used only if use_ddp_launch is False.
    """
    if "MASTER_ADDR" not in os.environ:
        os.environ["MASTER_ADDR"] = (
            "localhost" if master_addr is None else str(master_addr)
        )

    if "MASTER_PORT" not in os.environ:
        os.environ["MASTER_PORT"] = "12354" if master_port is None else str(master_port)

    if use_ddp_launch is False:
        dist.init_process_group("nccl", rank=rank, world_size=world_size)
        torch.cuda.set_device(rank)
    else:
        dist.init_process_group("nccl")


def cleanup_dist():
    dist.destroy_process_group()


def get_world_size():
    if "WORLD_SIZE" in os.environ:
        return int(os.environ["WORLD_SIZE"])
    if dist.is_available() and dist.is_initialized():
        return dist.get_world_size()
    else:
        return 1


def get_rank():
    if "RANK" in os.environ:
        return int(os.environ["RANK"])
    elif dist.is_available() and dist.is_initialized():
        return dist.get_rank()
    else:
        return 0


def get_local_rank():
    return int(os.environ.get("LOCAL_RANK", 0))



from gxl_ai_utils.utils import utils_file
"""
从问题text.scp, 回答预测的text.scp和真实回答text.scp,分别得到 真实和预测的 问答jsonl文件，用来直观展示
"""


Q_path = "/home/node54_tmpdata/xlgeng/chat_data/shards_test/text_Q"
A_path_hyp = "/home/node54_tmpdata/yzli/wenet_whisper_finetune/examples/wenetspeech/whisper/exp/qwen2_multi_task_6gpus_gxl_adapter_init_asr-sot_whisper/chat_epoch_9.pt_chunk-1_ctc0_reverse0.0_blankpenalty0.0_lengthpenalty0.0/llmasr_decode/text"
A_path_ref = "/home/node54_tmpdata/xlgeng/chat_data/shards_test/text_A"

Q_dict = utils_file.load_dict_from_scp(Q_path)
A_hyp = utils_file.load_dict_from_scp(A_path_hyp)
A_ref = utils_file.load_dict_from_scp(A_path_ref)
dict_list_hyp = []
dict_list_ref = []
jsonl_hyp_path = "/home/node54_tmpdata/xlgeng/chat_data/shards_test/test_hyp.jsonl"
jsonl_ref_path = "/home/node54_tmpdata/xlgeng/chat_data/shards_test/test_ref.jsonl"
for key, item in Q_dict.items():
    if key not in A_hyp or key not in A_ref:
        utils_file.logging_warning("warning: key not found: ", key)
        continue
    a_hyp_item = A_hyp[key]
    a_ref_item = A_ref[key]
    dict_item_hyp = {
        "key": key,
        "Q": item,
        "A": a_hyp_item,
    }
    dict_item_ref = {
        "key": key,
        "Q": item,
        "A": a_ref_item,
    }
    dict_list_hyp.append(dict_item_hyp)
    dict_list_ref.append(dict_item_ref)

utils_file.write_dict_list_to_jsonl(dict_list_hyp, jsonl_hyp_path)
utils_file.write_dict_list_to_jsonl(dict_list_ref, jsonl_ref_path)


import os.path
import random
import sys
sys.path.insert(0, '../../../')
from gxl_ai_utils.utils import utils_file

def split_file(input_file, num_parts):
    # utils_file.makedir_sil(output_dir)
    with open(input_file, 'rb') as f:
        f.seek(0, 2)  # 将文件指针移动到文件末尾
        file_size = f.tell()  # 获取文件大小
        chunk_size = file_size // num_parts  # 计算每一份的大小
        utils_file.logging_limit_print(f'chunk_size:{chunk_size}')
        f.seek(0)  # 将文件指针移回文件开头
        for i in range(num_parts):
            part_file = f"{input_file}_{i}.gxl_part"
            utils_file.logging_limit_print(f'part_file:{part_file}')
            with open(part_file, 'wb') as part:
                if i == num_parts - 1:  # 最后一部分可能会大一些，处理文件大小不能整除的情况
                    data = f.read()
                else:
                    data = f.read(chunk_size)
                part.write(data)


def combine_files(input_file, old_input_file, num_parts):
    output_file = input_file
    with open(output_file, 'wb') as out:
        for i in range(num_parts):
            part_file = f"{old_input_file}_{i}.gxl_part"
            print(part_file)
            with open(part_file, 'rb') as part:
                data = part.read()
                out.write(data)

def download_parts(old_input_file, output_dir,parts_num, num_thread=28):
    file_list = []
    for i in range(parts_num):
        file_list.append(f"{old_input_file}_{i}.gxl_part")
    fake_path = utils_file.do_get_fake_file()
    random.shuffle(file_list)
    utils_file.write_list_to_file(file_list, fake_path)
    utils_file.do_sync_files_download_data_multi_thread(
        file_list_path=fake_path,
        username="root",
        password="Fy!mATB@QE",
        remote_host="139.210.101.41",
        local_directory=output_dir,
        num_thread=num_thread
    )


if __name__ == "__main__":
    input_file = "/mnt/sfs/asr/code/wenet_undersdand_and_speech_xlgeng/examples/wenetspeech/whisper/exp/epoch_16_with_asr-chat_full_data_50percent_pureX/step_26249.pt"
    input_file = "/mnt/sfs/asr/code/wenet_undersdand_and_speech_xlgeng/examples/wenetspeech/whisper/exp/epoch21_cosyvoice1_new-set_token_1w_plus-multi_task/step_49999.pt"
    output_dir = input_file.replace('.pt', '')
    num_parts = 100  # 要分割的份数
    # split_file(input_file, num_parts)
    output_lab_dir = '/home/work_nfs16/asr_data/ckpt/understanding_model/epoch21_cosyvoice1_new/step_49999'
    utils_file.makedir_sil(output_lab_dir)
    download_parts(input_file,output_lab_dir,num_parts, num_thread=5)
    input_file_lab = output_lab_dir+".pt"
    old_input_file = f'{output_lab_dir}/{os.path.basename(input_file)}'
    combine_files(input_file_lab, old_input_file, num_parts)



import sys

import tqdm

from gxl_ai_utils.utils import utils_file



# 得到命令行参数
wav_scp_path = sys.argv[1]
text_scp_path = sys.argv[2]
output_dir = sys.argv[3]

utils_file.do_inference_paraformer(

)




import os
import shutil

import torch
import torchaudio
import yaml
from gxl_ai_utils.utils import utils_file
from torch.utils.data import DataLoader

from wenet.dataset.dataset import Dataset
config_dict = yaml.load(open("train.yaml", "r"), Loader=yaml.FullLoader)
dataset_conf = config_dict['dataset_conf']

test_dataset = Dataset("shard",
                       "/mnt/sfs/asr/update_data/asr_chat_wenetspeech_enhance_2025-1-24/shards_list.txt",
                       # "./tmp.list",
                       {},
                       None,
                       dataset_conf,
                       partition=False)

test_data_loader = DataLoader(test_dataset, batch_size=1, num_workers=10)

time_now = utils_file.do_get_now_time()
output_dir = "/mnt/sfs/asr/update_data/raw_data/asr_chat_znlin_2025-1-24"
utils_file.makedir(output_dir)
utils_file.makedir(f'{output_dir}/wav')
text_scp_path = os.path.join(output_dir, "text.scp")
text_f = open(text_scp_path, 'w', encoding='utf-8')
wav_scp_path = os.path.join(output_dir, "wav.scp")
wav_f = open(wav_scp_path, 'w', encoding='utf-8')

for i, batch in enumerate(test_data_loader):
    sorted_key, sorted_txt, sorted_wav = batch
    sorted_key = [k[0] for k in sorted_key]
    sorted_txt = [t[0] for t in sorted_txt]
    sorted_wav = [w[0] for w in sorted_wav]
    print(f'i: {i}')
    for key, txt, wav in zip(sorted_key, sorted_txt, sorted_wav):
        text_f.write(f'{key}\t{txt}\n')
        wav_path = os.path.join(f'{output_dir}/wav', f'{key}.wav')
        # torchaudio.save(wav_path, wav, 16000)
        with open(wav_path, "wb") as wav_file:
            wav_file.write(wav)
        wav_f.write(f'{key}\t{wav_path}\n')
        if i % 10000 == 0:
            text_f.flush()
            wav_f.flush()
time_elapsed = utils_file.do_get_elapsed_time(time_now)
print(f'time_elapsed: {time_elapsed}')


import re

from os import PathLike
from typing import Dict, List, Optional, Union
from wenet.utils.file_utils import read_symbol_table, read_non_lang_symbols
from wenet.text.base_tokenizer import BaseTokenizer


class CharTokenizer(BaseTokenizer):

    def __init__(
        self,
        symbol_table: Union[str, PathLike, Dict],
        non_lang_syms: Optional[Union[str, PathLike, List]] = None,
        split_with_space: bool = False,
        connect_symbol: str = '',
        unk='<unk>',
    ) -> None:
        self.non_lang_syms_pattern = None
        if non_lang_syms is not None:
            self.non_lang_syms_pattern = re.compile(
                r"(\[[^\[\]]+\]|<[^<>]+>|{[^{}]+})")
        if not isinstance(symbol_table, Dict):
            self._symbol_table = read_symbol_table(symbol_table)
        else:
            # symbol_table = {"我": 1, "是": 2, "{NOISE}": 3}
            self._symbol_table = symbol_table
        if not isinstance(non_lang_syms, List):
            self.non_lang_syms = read_non_lang_symbols(non_lang_syms)
        else:
            # non_lang_syms=["{NOISE}"]
            self.non_lang_syms = non_lang_syms
        self.char_dict = {v: k for k, v in self._symbol_table.items()}
        self.split_with_space = split_with_space
        self.connect_symbol = connect_symbol
        self.unk = unk

    def text2tokens(self, line: str) -> List[str]:
        line = line.strip()
        if self.non_lang_syms_pattern is not None:
            parts = self.non_lang_syms_pattern.split(line.upper())
            parts = [w for w in parts if len(w.strip()) > 0]
        else:
            parts = [line]

        tokens = []
        for part in parts:
            if part in self.non_lang_syms:
                tokens.append(part)
            else:
                if self.split_with_space:
                    part = part.split(" ")
                for ch in part:
                    if ch == ' ':
                        ch = "▁"
                    tokens.append(ch)
        return tokens

    def tokens2text(self, tokens: List[str]) -> str:
        return self.connect_symbol.join(tokens)

    def tokens2ids(self, tokens: List[str]) -> List[int]:
        ids = []
        for ch in tokens:
            if ch in self._symbol_table:
                ids.append(self._symbol_table[ch])
            elif self.unk in self._symbol_table:
                ids.append(self._symbol_table[self.unk])
        return ids

    def ids2tokens(self, ids: List[int]) -> List[str]:
        content = [self.char_dict[w] for w in ids]
        return content

    def vocab_size(self) -> int:
        return len(self.char_dict)

    @property
    def symbol_table(self) -> Dict[str, int]:
        return self._symbol_table



from gxl_ai_utils.utils import utils_file

input_dir = "/mnt/disk1/yhdai/data_zipformer/batch03"
dir_path_list, _ = utils_file.do_listdir(input_dir, return_path=True
                                         )
runner = utils_file.GxlDynamicThreadPool()
for dir_path in dir_path_list:
    runner.add_task(utils_file.do_compress_directory_to_tar_gz, [dir_path])
runner.run()



#!/usr/bin/env python3
# Copyright      2023  Xiaomi Corp.        (authors: Fangjun Kuang)

from pathlib import Path

import graphviz
import kaldifst
from prepare_lang import Lexicon, make_lexicon_fst_with_silence


def test_yesno():
    lang_dir = "/Users/fangjun/open-source/icefall/egs/yesno/ASR/data/lang_phone"
    if not Path(lang_dir).is_dir():
        print(f"{lang_dir} does not exist! Skip testing")
        return

    lexicon = Lexicon(lang_dir)

    L = make_lexicon_fst_with_silence(lexicon)

    isym = kaldifst.SymbolTable()
    for i, token in lexicon.id2token.items():
        isym.add_symbol(symbol=token, key=i)

    osym = kaldifst.SymbolTable()
    for i, word in lexicon.id2word.items():
        osym.add_symbol(symbol=word, key=i)

    L.input_symbols = isym
    L.output_symbols = osym
    fst_dot = kaldifst.draw(L, acceptor=False, portrait=True)
    source = graphviz.Source(fst_dot)
    source.render(outfile="L_yesno.pdf")
    # See the link below to visualize the above PDF
    # https://t.ly/jMfXW


def main():
    test_yesno()


if __name__ == "__main__":
    main()



from gxl_ai_utils.utils import utils_file


input_data_list_path = "/home/work_nfs8/xlgeng/new_workspace/wenet_gxl_salmonn/examples/aishell/salmonn/data_list/gxl_all_new_wenetspeech_fix.list"
path_list = utils_file.load_list_file_clean(input_data_list_path)

utils_file.logging_print("path_list len: {}".format(len(path_list)))
def little_func(path_list, output_root_dir):
    for path_i in utils_file.tqdm(path_list):
        """"""




import os
import sys
import re

from gxl_ai_utils.utils import utils_file

import jieba
import tqdm


def get_tag_from_str(input_str):
    # 使用正则表达式提取所有 <> 中的内容
    matches = re.findall(r'<.*?>', input_str)
    # 输出结果
    if len(matches)==0:
        return '<--no_tag-->'
    return matches[1].upper()
def do_compute_acc(ref_file, hyp_file, output_dir):
    """
    计算正确率和错误率，并列出混淆矩阵
    :return:
    """
    utils_file.makedir_sil(output_dir)
    utils_file.logging_info('开始计算正确率，acc')
    acc_path = os.path.join(output_dir, 'acc')
    hyp_dict = utils_file.load_dict_from_scp(hyp_file)
    ref_dict = utils_file.load_dict_from_scp(ref_file)
    tag_hyp_dict = {}
    for key, value in hyp_dict.items():
        tag_hyp_dict[key] = get_tag_from_str(value)
    tag_ref_dict = {}
    for key, value in ref_dict.items():
        tag_ref_dict[key] = get_tag_from_str(value)
    output_acc_f = open(acc_path, 'w', encoding='utf-8')
    same_num = 0
    all_tags = set()
    # utils_file.print_dict(utils_file.do_get_random_subdict(tag_ref_dict, 10))
    for tag in tag_ref_dict.values():
        all_tags.add(tag)
    # for tag in tag_hyp_dict.values():
    #     all_tags.add(tag)
    labels = sorted(list(all_tags))

    utils_file.logging_info(f'标签种类为：{labels}')
    # labels = [style_convert_dict.get(label, label) for label in labels]
    utils_file.logging_info(f'标签种类为：{labels}')
    num_classes = len(labels)
    matrix = [[0] * num_classes for _ in range(num_classes)]
    for key, hyp_tags in tag_hyp_dict.items():
        if key not in tag_ref_dict:
            continue
        ref_tags = tag_ref_dict[key]
        # 判断标签是否相同
        if ref_tags == hyp_tags:
            if_same = True
            same_num += 1
            index = labels.index(ref_tags)
            matrix[index][index] += 1
        else:
            if_same = False
            # 标签不同的情况，找到真实标签和预测标签对应的索引，在相应的位置加1
            ref_index = labels.index(ref_tags)
            if hyp_tags in labels:
                hyp_index = labels.index(hyp_tags)
                matrix[ref_index][hyp_index] += 1
        # 向文件写入数据
        output_acc_f.write(f"key: {key}\n")
        output_acc_f.write(f"ref_tag: {ref_tags}\n")
        output_acc_f.write(f"hyp_tag: {hyp_tags}\n")
        output_acc_f.write(f"if_same: {if_same}\n")
        output_acc_f.write("\n")  # 添加空行分隔不同的条目
    acc_num = same_num / len(tag_hyp_dict)
    output_acc_f.write(f'正确率为：{acc_num}')
    output_acc_f.flush()
    output_acc_f.close()
    # utils_file.copy_file(acc_path, os.path.join(output_dir2, 'acc'), use_shell=True)
    # figure_path = os.path.join(output_dir, 'confusion_matrix.png')
    # do_showing_confusion_matrix(labels, matrix, output_fig_path=figure_path)
    # utils_file.copy_file(figure_path, os.path.join(output_dir2, 'confusion_matrix.png'))
    return {
        'acc': acc_num,
        # 'wer': wer_all,
    }

res_text_path = "/home/work_nfs15/asr_data/ckpt/understanding_model/epoch_16_with_asr-chat_full_data_50percent_pureX/test_step_26249.pt/age_gender/age_gender_only_age/llmasr_decode/text"
res_text_path = "/home/work_nfs15/asr_data/ckpt/understanding_model/epoch_16_with_asr-chat_full_data_50percent_pureX/test_step_26249.pt/asr_age_gender/llmasr_decode/text"
lab_data_list_path = "/home/work_nfs15/asr_data/data/test_sets_format_3000/age_gender/age_gender_only_gender/data.list"
text_lab_path = "/home/work_nfs15/asr_data/data/test_sets_format_3000/age_gender/age_gender_only_gender/text"
dict_list = utils_file.load_dict_list_from_jsonl(lab_data_list_path)
dict_text = {}
for dict_i in dict_list:
    txt = dict_i['txt']
    dict_text[dict_i['key']] = txt
utils_file.write_dict_to_scp(dict_text, text_lab_path)
res = do_compute_acc(text_lab_path, res_text_path, "./tmp")
print(res)
    